@@ -9,15 +9,18 @@
  */
 
 #include "align_stack.h"
+
 #include "align_tab_column.h"
 #include "chunk_list.h"
 #include "indent.h"
-#include "space.h"
 #include "prototypes.h"
+#include "space.h"
 #include "uncrustify.h"
+//#include "unc_tools.h"                   // to get stackID and get_A_Number()
 
 #include <limits>
 
+
 using namespace uncrustify;
 
 using std::numeric_limits;
@@ -25,10 +28,13 @@ using std::numeric_limits;
 
 void AlignStack::Start(size_t span, int thresh)
 {
+   //stackID = get_A_Number();   // for debugging purpose only
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu, ", __func__, __LINE__, stackID);
    LOG_FMT(LAS, "AlignStack::Start(%d): span is %zu, thresh is %d\n", __LINE__, span, thresh);
 
    m_aligned.Reset();
    m_skipped.Reset();
+
    if (thresh > 0)
    {
       m_absolute_thresh = false;
@@ -39,7 +45,6 @@ void AlignStack::Start(size_t span, int thresh)
       m_absolute_thresh = true;
       m_thresh          = -thresh;
    }
-
    m_span        = span;
    m_min_col     = numeric_limits<size_t>::max();
    m_max_col     = 0;
@@ -54,11 +59,12 @@ void AlignStack::Start(size_t span, int thresh)
 
 void AlignStack::ReAddSkipped()
 {
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu\n", __func__, __LINE__, stackID);
+
    if (m_skipped.Empty())
    {
       return;
    }
-
    // Make a copy of the ChunkStack and clear m_skipped
    m_scratch.Set(m_skipped);
    m_skipped.Reset();
@@ -77,13 +83,17 @@ void AlignStack::ReAddSkipped()
 
 void AlignStack::Add(chunk_t *start, size_t seqnum)
 {
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu, for '%s', ", __func__, __LINE__, stackID, start->text());
    LOG_FUNC_ENTRY();
+
+   LOG_FMT(LAS, "AlignStack::%s(%d): Candidate is '%s': orig_line is %zu, column is %zu, type is %s, level is %zu\n",
+           __func__, __LINE__, start->text(), start->orig_line, start->column, get_token_name(start->type), start->level);
+
    // Assign a seqnum if needed
    if (seqnum == 0)
    {
       seqnum = m_seqnum;
    }
-
    m_last_added = 0;
 
    // Threshold check should begin after
@@ -149,35 +159,41 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
    {
       m_star_style = SS_INCLUDE;
    }
-
+   LOG_FMT(LAS, "AlignStack::%s(%d): m_star_style is %s\n",
+           __func__, __LINE__, get_StarStyle_name(m_star_style));
    // Find ref. Back up to the real item that is aligned.
    chunk_t *prev = start;
+
    while (  (prev = chunk_get_prev(prev)) != nullptr
          && (chunk_is_ptr_operator(prev) || chunk_is_token(prev, CT_TPAREN_OPEN)))
    {
       // do nothing - we want prev when this exits
    }
+
    if (prev == nullptr)
    {
       return;
    }
    chunk_t *ref = prev;
+
    if (chunk_is_newline(ref))
    {
       ref = chunk_get_next(ref);
    }
-
    // Find the item that we are going to align.
    chunk_t *ali = start;
+
    if (m_star_style != SS_IGNORE)
    {
       // back up to the first '*' or '^' preceding the token
       chunk_t *tmp_prev = chunk_get_prev(ali);
+
       while (chunk_is_star(tmp_prev) || chunk_is_msref(tmp_prev))
       {
          ali      = tmp_prev;
          tmp_prev = chunk_get_prev(ali);
       }
+
       if (chunk_is_token(tmp_prev, CT_TPAREN_OPEN))
       {
          ali      = tmp_prev;
@@ -187,10 +203,12 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
          // chunk_get_prev(ali, nav_e::ALL) to prev here, but that stored value is overwritten before it can be used.
       }
    }
+
    if (m_amp_style != SS_IGNORE)
    {
       // back up to the first '&' preceding the token
       chunk_t *tmp_prev = chunk_get_prev(ali);
+
       while (chunk_is_addr(tmp_prev))
       {
          ali      = tmp_prev;
@@ -198,7 +216,6 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
       }
    }
 
-
    // Tighten down the spacing between ref and start
    if (!options::align_keep_extra_space())
    {
@@ -206,16 +223,19 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
       chunk_t *tmp    = ref;
       LOG_FMT(LAS, "AlignStack::%s(%d): tmp_col is %zu\n",
               __func__, __LINE__, tmp_col);
+
       while (tmp != nullptr && tmp != start)
       {
          chunk_t *next = chunk_get_next(tmp);
+
          if (next != nullptr)
          {
-            LOG_FMT(LAS, "AlignStack::%s(%d): orig_line is %zu, orig_col is %zu, text() '%s', type is %s\n",
-                    __func__, __LINE__, next->orig_line, next->orig_col, next->text(), get_token_name(next->type));
+            LOG_FMT(LAS, "AlignStack::%s(%d): next->orig_line is %zu, orig_col is %zu, text() '%s', level is %zu, type is %s\n",
+                    __func__, __LINE__, next->orig_line, next->orig_col, next->text(), next->level, get_token_name(next->type));
             tmp_col += space_col_align(tmp, next);
-            LOG_FMT(LAS, "AlignStack::%s(%d): column is %zu, tmp_col is %zu\n",
-                    __func__, __LINE__, next->column, tmp_col);
+            LOG_FMT(LAS, "AlignStack::%s(%d): next->column is %zu, level is %zu, tmp_col is %zu\n",
+                    __func__, __LINE__, next->column, next->level, tmp_col);
+
             if (next->column != tmp_col)
             {
                LOG_FMT(LAS, "AlignStack::%s(%d): Call align_to_column\n", __func__, __LINE__);
@@ -238,20 +258,21 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
       {
          m_nl_seqnum = seqnum;
       }
-
       // Set the column adjust and gap
       size_t col_adj = 0; // Amount the column is shifted for 'dangle' mode
       size_t gap     = 0;
+
       if (ref != ali)
       {
          gap = ali->column - (ref->column + ref->len());
       }
-
       chunk_t *tmp = ali;
+
       if (chunk_is_token(tmp, CT_TPAREN_OPEN))
       {
          tmp = chunk_get_next(tmp);
       }
+
       if (  (chunk_is_star(tmp) && m_star_style == SS_DANGLE)
          || (chunk_is_addr(tmp) && m_amp_style == SS_DANGLE)
          || (chunk_is_nullable(tmp) && (m_star_style == SS_DANGLE))
@@ -260,7 +281,6 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
          col_adj = start->column - ali->column;
          gap     = start->column - (ref->column + ref->len());
       }
-
       // See if this pushes out the max_col
       const size_t endcol = ali->column + col_adj
                             + (gap < m_gap ? m_gap - gap : 0);
@@ -271,8 +291,17 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
       m_aligned.Push_Back(ali, seqnum);
       m_last_added = 1;
 
-      LOG_FMT(LAS, "AlignStack::%s(%d): Add-[%s]: ali->orig_line is %zu, ali->column is %zu, type is %s\n",
-              __func__, __LINE__, ali->text(), ali->orig_line, ali->column, get_token_name(ali->type));
+      // Issue #2278
+      if (chunk_is_token(ali, CT_PTR_TYPE))
+      {
+         LOG_FMT(LAS, "AlignStack::%s(%d): Add-[%s][%s]: ali->orig_line is %zu, column is %zu, type is %s, level is %zu\n",
+                 __func__, __LINE__, ali->text(), start->text(), ali->orig_line, ali->column, get_token_name(ali->type), ali->level);
+      }
+      else
+      {
+         LOG_FMT(LAS, "AlignStack::%s(%d): Add-[%s]: ali->orig_line is %zu, column is %zu, type is %s, level is %zu\n",
+                 __func__, __LINE__, ali->text(), ali->orig_line, ali->column, get_token_name(ali->type), ali->level);
+      }
       LOG_FMT(LAS, "AlignStack::%s(%d):    ali->align.col_adj is %d, ref '%s', endcol is %zu\n",
               __func__, __LINE__, ali->align.col_adj, ref->text(), endcol);
 
@@ -300,9 +329,10 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
       }
       else
       {
-         LOG_FMT(LAS, "AlignStack::Add-aligned [%zu/%zu/%zu]: line %zu, col %zu : col %zu <= %zu - min_col %zu\n",
-                 seqnum, m_nl_seqnum, m_seqnum,
-                 ali->orig_line, ali->column, endcol, m_max_col, m_min_col);
+         LOG_FMT(LAS, "AlignStack::%s(%d): Add-aligned: seqnum is %zu, m_nl_seqnum is %zu, m_seqnum is %zu\n",
+                 __func__, __LINE__, seqnum, m_nl_seqnum, m_seqnum);
+         LOG_FMT(LAS, "AlignStack::%s(%d):    ali->orig_line is %zu, ali->column is %zu, max_col old is %zu, new is %zu, m_min_col is %zu\n",
+                 __func__, __LINE__, ali->orig_line, ali->column, m_max_col, endcol, m_min_col);
       }
    }
    else
@@ -315,23 +345,26 @@ void AlignStack::Add(chunk_t *start, size_t seqnum)
               seqnum, m_nl_seqnum, m_seqnum,
               start->orig_line, start->column, m_max_col, m_thresh);
    }
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu, ", __func__, __LINE__, stackID);
 } // AlignStack::Add
 
 
 void AlignStack::NewLines(size_t cnt)
 {
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu\n", __func__, __LINE__, stackID);
+
    if (m_aligned.Empty())
    {
-      LOG_FMT(LAS, "AlignStack::Newlines(%d): is empty\n", __LINE__);
+      LOG_FMT(LAS, "AlignStack::Newlines(%d): nothing to do, is empty\n", __LINE__);
       return;
    }
-
    m_seqnum += cnt;
    LOG_FMT(LAS, "AlignStack::Newlines(%d): m_seqnum is %zu, m_nl_seqnum is %zu, m_span is %zu, \n",
            __LINE__, m_seqnum, m_nl_seqnum, m_span);
+
    if (m_seqnum > (m_nl_seqnum + m_span))
    {
-      LOG_FMT(LAS, "AlignStack::Newlines(%d): cnt is %zu, >\n", __LINE__, cnt);
+      LOG_FMT(LAS, "AlignStack::Newlines(%d): cnt is %zu\n", __LINE__, cnt);
       Flush();
    }
 }
@@ -339,14 +372,22 @@ void AlignStack::NewLines(size_t cnt)
 
 void AlignStack::Flush()
 {
-   LOG_FMT(LAS, "AlignStack::%s(%d): m_aligned.Len() is %zu\n", __func__, __LINE__, m_aligned.Len());
-   LOG_FMT(LAS, "   (min is %zu, max is %zu)\n", m_min_col, m_max_col);
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu\n", __func__, __LINE__, stackID);
+   LOG_FMT(LAS, "AlignStack::%s(%d): Len() is %zu\n",
+           __func__, __LINE__, Len());
 
-   if (m_aligned.Len() == 1)
+   if (Len() > 0)
+   {
+      LOG_FMT(LAS, "   (min is %zu, max is %zu)\n",
+              m_min_col, m_max_col);
+   }
+
+   if (Len() == 1)
    {
       // check if we have *one* typedef in the line
       chunk_t *pc   = m_aligned.Get(0)->m_pc;
       chunk_t *temp = chunk_get_prev_type(pc, CT_TYPEDEF, pc->level);
+
       if (temp != nullptr)
       {
          if (pc->orig_line == temp->orig_line)
@@ -356,18 +397,25 @@ void AlignStack::Flush()
          }
       }
    }
-
    m_last_added = 0;
    m_max_col    = 0;
 
+   for (size_t idx = 0; idx < Len(); idx++)
+   {
+      chunk_t *pc = m_aligned.Get(idx)->m_pc;
+      LOG_FMT(LAS, "AlignStack::%s(%d): idx is %zu, pc->text() is '%s', pc->align.col_adj is %d\n",
+              __func__, __LINE__, idx, pc->text(), pc->align.col_adj);
+   }
+
    // Recalculate the max_col - it may have shifted since the last Add()
-   for (size_t idx = 0; idx < m_aligned.Len(); idx++)
+   for (size_t idx = 0; idx < Len(); idx++)
    {
       chunk_t *pc = m_aligned.Get(idx)->m_pc;
 
       // Set the column adjust and gap
       size_t col_adj = 0;
       size_t gap     = 0;
+
       if (pc != pc->align.ref)
       {
          gap = pc->column - (pc->align.ref->column + pc->align.ref->len());
@@ -376,19 +424,23 @@ void AlignStack::Flush()
       if (m_star_style == SS_DANGLE)
       {
          chunk_t *tmp = (chunk_is_token(pc, CT_TPAREN_OPEN)) ? chunk_get_next(pc) : pc;
+
          if (chunk_is_ptr_operator(tmp))
          {
             col_adj = pc->align.start->column - pc->column;
             gap     = pc->align.start->column - (pc->align.ref->column + pc->align.ref->len());
          }
       }
+
       if (m_right_align)
       {
          // Adjust the width for signed numbers
          size_t start_len = pc->align.start->len();
+
          if (pc->align.start->type == CT_NEG)
          {
             chunk_t *next = chunk_get_next(pc->align.start);
+
             if (chunk_is_token(next, CT_NUMBER))
             {
                start_len += next->len();
@@ -396,36 +448,46 @@ void AlignStack::Flush()
          }
          col_adj += start_len;
       }
-
       pc->align.col_adj = col_adj;
 
       // See if this pushes out the max_col
       const size_t endcol = pc->column + col_adj
                             + (gap < m_gap ? m_gap - gap : 0);
+
       if (endcol > m_max_col)
       {
          m_max_col = endcol;
       }
    }
 
-   if (options::align_on_tabstop() && m_aligned.Len() > 1)
+   if (  options::align_on_tabstop()
+      && Len() > 1)
    {
       m_max_col = align_tab_column(m_max_col);
    }
-
-   LOG_FMT(LAS, "AlignStack::%s(%d): m_aligned.Len() is %zu\n",
-           __func__, __LINE__, m_aligned.Len());
+   LOG_FMT(LAS, "AlignStack::%s(%d): Len() is %zu\n",
+           __func__, __LINE__, Len());
 
    const ChunkStack::Entry *ce = nullptr;
-   for (size_t idx = 0; idx < m_aligned.Len(); idx++)
+
+   for (size_t idx = 0; idx < Len(); idx++)
+   {
+      ce = m_aligned.Get(idx);
+      LOG_FMT(LAS, "AlignStack::%s(%d): idx is %zu, ce->m_pc->text() is '%s', ce->m_pc->align.col_adj is %d\n",
+              __func__, __LINE__, idx, ce->m_pc->text(), ce->m_pc->align.col_adj);
+   }
+
+   for (size_t idx = 0; idx < Len(); idx++)
    {
       ce = m_aligned.Get(idx);
       chunk_t      *pc = ce->m_pc;
 
       const size_t tmp_col = m_max_col - pc->align.col_adj;
+
       if (idx == 0)
       {
-         if (m_skip_first && pc->column != tmp_col)
+         if (  m_skip_first
+            && pc->column != tmp_col)
          {
             LOG_FMT(LAS, "AlignStack::%s(%d): orig_line is %zu, orig_col is %zu, dropping first item due to skip_first\n",
                     __func__, __LINE__, pc->orig_line, pc->orig_col);
@@ -451,6 +513,7 @@ void AlignStack::Flush()
    }
 
    size_t last_seqnum = 0;
+
    if (ce != nullptr)
    {
       last_seqnum = ce->m_seqnum;
@@ -474,6 +537,7 @@ void AlignStack::Flush()
             m_skipped.Zap(idx);
          }
       }
+
       m_skipped.Collapse();
 
       ReAddSkipped(); // Add all items from the skipped list
@@ -483,6 +547,7 @@ void AlignStack::Flush()
 
 void AlignStack::Reset()
 {
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu\n", __func__, __LINE__, stackID);
    m_aligned.Reset();
    m_skipped.Reset();
 }
@@ -490,12 +555,56 @@ void AlignStack::Reset()
 
 void AlignStack::End()
 {
+   //LOG_FMT(LAS, "AlignStack::%s(%d): stackID is %2zu\n", __func__, __LINE__, stackID);
+
    if (!m_aligned.Empty())
    {
       LOG_FMT(LAS, "AlignStack::End(%d):\n", __LINE__);
       Flush();
    }
-
    m_aligned.Reset();
    m_skipped.Reset();
 }
+
+
+size_t AlignStack::Len()
+{
+   return(m_aligned.Len());
+}
+
+
+void AlignStack::Debug()
+{
+   for (size_t idx = 0; idx < Len(); idx++)
+   {
+      chunk_t *pc = m_aligned.Get(idx)->m_pc;
+
+      if (chunk_is_token(pc, CT_PTR_TYPE))
+      {
+         LOG_FMT(LAS, "AlignStack::%s(%d): idx is %zu, [%s][%s]: orig_line is %zu, type is %s, level is %zu\n",
+                 __func__, __LINE__, idx, pc->text(), pc->next->text(), pc->orig_line, get_token_name(pc->type), pc->level);
+      }
+      else
+      {
+         LOG_FMT(LAS, "AlignStack::%s(%d): idx is %zu, [%s]: orig_line is %zu, type is %s, level is %zu\n",
+                 __func__, __LINE__, idx, pc->text(), pc->orig_line, get_token_name(pc->type), pc->level);
+      }
+   }
+}
+
+
+const char *AlignStack::get_StarStyle_name(StarStyle star_style)
+{
+   switch (star_style)
+   {
+   case StarStyle::SS_IGNORE:
+      return("SS_IGNORE");
+
+   case StarStyle::SS_INCLUDE:
+      return("SS_INCLUDE");
+
+   case StarStyle::SS_DANGLE:
+      return("SS_DANGLE");
+   }
+   return("?????");
+} // get_StarStyle_name
