@@ -9,24 +9,26 @@
  */
 
 #include "brace_cleanup.h"
+
 #include "chunk_list.h"
-#include "frame_list.h"
 #include "flag_parens.h"
+#include "frame_list.h"
 #include "indent.h"
 #include "keywords.h"
-#include "language_tools.h"
 #include "lang_pawn.h"
+#include "language_tools.h"
+#include "log_rules.h"
 #include "logger.h"
 #include "prototypes.h"
+#include "unc_ctype.h"
 #include "uncrustify.h"
 #include "uncrustify_types.h"
-#include "unc_ctype.h"
 
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-#include <stdexcept>
 #include <iostream>
+#include <stdexcept>
 
 using namespace uncrustify;
 
@@ -120,19 +122,19 @@ static size_t preproc_start(ParseFrame &frm, chunk_t *pc)
    const size_t pp_level = cpd.pp_level;
 
    chunk_t      *next = chunk_get_next_ncnl(pc);
+
    if (next == nullptr)
    {
       return(pp_level);
    }
-
    // Get the type of preprocessor and handle it
    cpd.in_preproc = next->type;
+
    // If we are not in a define, check for #if, #else, #endif, etc
    if (cpd.in_preproc != CT_PP_DEFINE)
    {
       return(fl_check(frm, pc));
    }
-
    // else push the frame stack
    fl_push(frm);
 
@@ -142,8 +144,7 @@ static size_t preproc_start(ParseFrame &frm, chunk_t *pc)
    frm.brace_level = 1;
 
    // TODO: not sure about the next 3 lines
-   chunk_t tmp{};
-   frm.push(tmp);
+   frm.push(nullptr);
    frm.top().type = CT_PP_DEFINE;
 
    return(pp_level);
@@ -154,11 +155,11 @@ static void print_stack(log_sev_t logsev, const char *str,
                         const ParseFrame &frm)
 {
    LOG_FUNC_ENTRY();
+
    if (!log_sev_on(logsev))
    {
       return;
    }
-
    log_fmt(logsev, "%s(%d): str is '%s'", __func__, __LINE__, str);
 
    for (size_t idx = 1; idx < frm.size(); idx++)
@@ -190,20 +191,19 @@ void brace_cleanup(void)
 
    ParseFrame frm{};
    chunk_t    *pc = chunk_get_head();
+
    while (pc != nullptr)
    {
       // Check for leaving a #define body
-      if (cpd.in_preproc != CT_NONE && (pc->flags & PCF_IN_PREPROC) == 0)
+      if (cpd.in_preproc != CT_NONE && !pc->flags.test(PCF_IN_PREPROC))
       {
          if (cpd.in_preproc == CT_PP_DEFINE)
          {
             // out of the #define body, restore the frame
             fl_pop(frm);
          }
-
          cpd.in_preproc = CT_NONE;
       }
-
       // Check for a preprocessor start
       const size_t pp_level = (chunk_is_token(pc, CT_PREPROC))
                               ? preproc_start(frm, pc)
@@ -215,6 +215,7 @@ void brace_cleanup(void)
          && chunk_is_token(pc, CT_NEWLINE))
       {
          pc = pawn_check_vsemicolon(pc);
+
          if (pc == nullptr)
          {
             return;
@@ -255,19 +256,20 @@ static bool maybe_while_of_do(chunk_t *pc)
    LOG_FUNC_ENTRY();
 
    chunk_t *prev = chunk_get_prev_ncnl(pc);
-   if (prev == nullptr || !(prev->flags & PCF_IN_PREPROC))
+
+   if (prev == nullptr || !prev->flags.test(PCF_IN_PREPROC))
    {
       return(false);
    }
 
    // Find the chunk before the preprocessor
-   while (prev != nullptr && (prev->flags & PCF_IN_PREPROC))
+   while (prev != nullptr && prev->flags.test(PCF_IN_PREPROC))
    {
       prev = chunk_get_prev_ncnl(prev);
    }
 
    if (  (chunk_is_token(prev, CT_VBRACE_CLOSE) || chunk_is_token(prev, CT_BRACE_CLOSE))
-      && prev->parent_type == CT_DO)
+      && get_chunk_parent_type(prev) == CT_DO)
    {
       return(true);
    }
@@ -344,6 +346,7 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
            __func__, __LINE__, pc->orig_line, get_token_name(pc->type), pc->text());
    LOG_FMT(LTOK, "%s(%d): frm.stmt_count is %zu, frm.expr_count is %zu\n",
            __func__, __LINE__, frm.stmt_count, frm.expr_count);
+
    if (  (frm.stmt_count == 0 || frm.expr_count == 0)
       && !chunk_is_semicolon(pc)
       && pc->type != CT_BRACE_CLOSE
@@ -351,10 +354,10 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
       && !chunk_is_str(pc, ")", 1)
       && !chunk_is_str(pc, "]", 1))
    {
-      chunk_flags_set(pc, PCF_EXPR_START | ((frm.stmt_count == 0) ? PCF_STMT_START : 0));
+      chunk_flags_set(pc, PCF_EXPR_START | ((frm.stmt_count == 0) ? PCF_STMT_START : PCF_NONE));
       LOG_FMT(LSTMT, "%s(%d): orig_line is %zu, 1.marked '%s' as %s, start stmt_count is %zu, expr_count is %zu\n",
               __func__, __LINE__, pc->orig_line, pc->text(),
-              (pc->flags & PCF_STMT_START) ? "stmt" : "expr", frm.stmt_count,
+              pc->flags.test(PCF_STMT_START) ? "stmt" : "expr", frm.stmt_count,
               frm.expr_count);
    }
    frm.stmt_count++;
@@ -387,6 +390,7 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
 
    // Check the progression of complex statements
    if (  frm.top().stage != brace_stage_e::NONE
+      && !chunk_is_token(pc, CT_AUTORELEASEPOOL)
       && check_complex_statements(frm, pc))
    {
       return;
@@ -405,7 +409,8 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
          cpd.consumed = true;
          close_statement(frm, pc);
       }
-      else if (language_is_set(LANG_PAWN) && chunk_is_token(pc, CT_BRACE_CLOSE))
+      else if (  language_is_set(LANG_PAWN)
+              && chunk_is_token(pc, CT_BRACE_CLOSE))
       {
          close_statement(frm, pc);
       }
@@ -426,6 +431,7 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
       {
          // TODO: fix enum hack
          set_chunk_type(pc, static_cast<c_token_t>(frm.top().type + 1));
+
          if (chunk_is_token(pc, CT_SPAREN_CLOSE))
          {
             frm.sparen_count--;
@@ -440,6 +446,7 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
                  __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
          LOG_FMT(LWARN, "%s(%d): (frm.top().type + 1) is %s\n",
                  __func__, __LINE__, get_token_name((c_token_t)(frm.top().type + 1)));
+
          if (  frm.top().type != CT_EOF
             && frm.top().type != CT_PP_DEFINE)
          {
@@ -458,6 +465,7 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
          // Copy the parent, update the parenthesis/brace levels
          set_chunk_parent(pc, frm.top().parent);
          frm.level--;
+
          if (  chunk_is_token(pc, CT_BRACE_CLOSE)
             || chunk_is_token(pc, CT_VBRACE_CLOSE)
             || chunk_is_token(pc, CT_MACRO_CLOSE))
@@ -473,7 +481,7 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
          // Pop the entry
          LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
                  __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-         frm.pop();
+         frm.pop(__func__, __LINE__);
          print_stack(LBCSPOP, "-Close  ", frm);
 
          // See if we are in a complex statement
@@ -528,15 +536,16 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
          handle_complex_close(frm, pc);
       }
    }
-
    // Get the parent type for brace and parenthesis open
-   c_token_t parent = pc->parent_type;
+   c_token_t parent = get_chunk_parent_type(pc);
+
    if (  chunk_is_token(pc, CT_PAREN_OPEN)
       || chunk_is_token(pc, CT_FPAREN_OPEN)
       || chunk_is_token(pc, CT_SPAREN_OPEN)
       || chunk_is_token(pc, CT_BRACE_OPEN))
    {
       chunk_t *prev = chunk_get_prev_ncnl(pc);
+
       if (prev != nullptr)
       {
          if (  chunk_is_token(pc, CT_PAREN_OPEN)
@@ -544,7 +553,18 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
             || chunk_is_token(pc, CT_SPAREN_OPEN))
          {
             // Set the parent for parenthesis and change parenthesis type
-            if (frm.top().stage != brace_stage_e::NONE)
+            if (  chunk_is_token(prev, CT_IF)
+               || chunk_is_token(prev, CT_ELSEIF)
+               || chunk_is_token(prev, CT_WHILE)
+               || chunk_is_token(prev, CT_DO)
+               || chunk_is_token(prev, CT_FOR)
+               || chunk_is_token(prev, CT_SWITCH)
+               || chunk_is_token(prev, CT_CATCH)
+               || chunk_is_token(prev, CT_SYNCHRONIZED)
+               || chunk_is_token(prev, CT_D_VERSION)
+               || chunk_is_token(prev, CT_D_VERSION_IF)
+               || chunk_is_token(prev, CT_D_SCOPE)
+               || chunk_is_token(prev, CT_D_SCOPE_IF))
             {
                set_chunk_type(pc, CT_SPAREN_OPEN);
                parent = frm.top().type;
@@ -586,7 +606,7 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
             // Carry through CT_ENUM parent in NS_ENUM (type, name) {
             else if (  chunk_is_token(prev, CT_FPAREN_CLOSE)
                     && language_is_set(LANG_OC)
-                    && prev->parent_type == CT_ENUM)
+                    && get_chunk_parent_type(prev) == CT_ENUM)
             {
                parent = CT_ENUM;
             }
@@ -612,19 +632,27 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
       || chunk_is_token(pc, CT_SQUARE_OPEN))
    {
       frm.level++;
-      if (chunk_is_token(pc, CT_BRACE_OPEN) || chunk_is_token(pc, CT_MACRO_OPEN))
+
+      if (  chunk_is_token(pc, CT_BRACE_OPEN)
+         || chunk_is_token(pc, CT_MACRO_OPEN))
       {
          // Issue #1813
          bool single = false;
-         if (pc->parent_type == CT_NAMESPACE)
+
+         if (get_chunk_parent_type(pc) == CT_NAMESPACE)
          {
             LOG_FMT(LBCSPOP, "%s(%d): parent_type is NAMESPACE\n",
                     __func__, __LINE__);
             chunk_t *tmp = frm.top().pc;
-            if (tmp != nullptr && tmp->parent_type == CT_NAMESPACE)
+
+            if (tmp != nullptr && get_chunk_parent_type(tmp) == CT_NAMESPACE)
             {
                LOG_FMT(LBCSPOP, "%s(%d): tmp->parent_type is NAMESPACE\n",
                        __func__, __LINE__);
+
+               log_rule_B("indent_namespace");
+               log_rule_B("indent_namespace_single_indent");
+
                if (  options::indent_namespace()
                   && options::indent_namespace_single_indent())
                {
@@ -635,7 +663,8 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
             }
          }
          LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s, parent_type is %s\n",
-                 __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type), get_token_name(pc->parent_type));
+                 __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type), get_token_name(get_chunk_parent_type(pc)));
+
          if (!single)
          {
             frm.brace_level++;
@@ -643,11 +672,61 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
                     __func__, __LINE__, frm.brace_level);
          }
       }
-      frm.push(*pc);
+      frm.push(pc, __func__, __LINE__);
       frm.top().parent = parent;
+      // set parent type
       set_chunk_parent(pc, parent);
    }
+   // Issue #2281
+   LOG_FMT(LBCSPOP, "%s(%d):\n", __func__, __LINE__);
 
+   if (  chunk_is_token(pc, CT_BRACE_OPEN)
+      && pc->parent_type == CT_SWITCH)
+   {
+      size_t idx = frm.size();
+      LOG_FMT(LBCSPOP, "%s(%d): idx is %zu\n",
+              __func__, __LINE__, idx);
+      chunk_t *saved = frm.at(idx - 2).pc;
+
+      if (saved != nullptr)
+      {
+         // set parent member
+         chunk_set_parent(pc, saved);
+      }
+   }
+
+   if (chunk_is_token(pc, CT_CASE))
+   {
+      LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, pc->orig_col is %zu\n",
+              __func__, __LINE__, pc->orig_line, pc->orig_col);
+      set_chunk_parent(pc, CT_SWITCH);
+      size_t idx = frm.size();
+      LOG_FMT(LBCSPOP, "%s(%d): idx is %zu\n",
+              __func__, __LINE__, idx);
+      chunk_t *saved = frm.at(idx - 2).pc;
+
+      if (saved != nullptr)
+      {
+         // set parent member
+         chunk_set_parent(pc, saved);
+      }
+   }
+
+   if (chunk_is_token(pc, CT_BREAK))
+   {
+      LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, pc->orig_col is %zu\n",
+              __func__, __LINE__, pc->orig_line, pc->orig_col);
+      size_t idx = frm.size();
+      LOG_FMT(LBCSPOP, "%s(%d): idx is %zu\n",
+              __func__, __LINE__, idx);
+      chunk_t *saved = frm.at(idx - 2).pc;
+
+      if (saved != nullptr)
+      {
+         // set parent member
+         chunk_set_parent(pc, saved);
+      }
+   }
    const pattern_class_e patcls = get_token_pattern_class(pc->type);
 
    /*
@@ -657,8 +736,8 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
     */
    if (patcls == pattern_class_e::BRACED)
    {
-      frm.push(*pc, (chunk_is_token(pc, CT_DO) ? brace_stage_e::BRACE_DO
-                     : brace_stage_e::BRACE2));
+      frm.push(pc, __func__, __LINE__, (chunk_is_token(pc, CT_DO) ? brace_stage_e::BRACE_DO
+                    : brace_stage_e::BRACE2));
       // "+ComplexBraced"
    }
    else if (patcls == pattern_class_e::PBRACED)
@@ -670,17 +749,17 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
          set_chunk_type(pc, CT_WHILE_OF_DO);
          bs = brace_stage_e::WOD_PAREN;
       }
-      frm.push(*pc, bs);
+      frm.push(pc, __func__, __LINE__, bs);
       // "+ComplexParenBraced"
    }
    else if (patcls == pattern_class_e::OPBRACED)
    {
-      frm.push(*pc, brace_stage_e::OP_PAREN1);
+      frm.push(pc, __func__, __LINE__, brace_stage_e::OP_PAREN1);
       // "+ComplexOpParenBraced");
    }
    else if (patcls == pattern_class_e::ELSE)
    {
-      frm.push(*pc, brace_stage_e::ELSEIF);
+      frm.push(pc, __func__, __LINE__, brace_stage_e::ELSEIF);
       // "+ComplexElse");
    }
 
@@ -691,10 +770,10 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
     *  - after '(' that has a parent type of CT_FOR
     */
    if (  chunk_is_token(pc, CT_SQUARE_OPEN)
-      || (chunk_is_token(pc, CT_BRACE_OPEN) && pc->parent_type != CT_ASSIGN)
+      || (chunk_is_token(pc, CT_BRACE_OPEN) && get_chunk_parent_type(pc) != CT_ASSIGN)
       || chunk_is_token(pc, CT_BRACE_CLOSE)
       || chunk_is_token(pc, CT_VBRACE_CLOSE)
-      || (chunk_is_token(pc, CT_SPAREN_OPEN) && pc->parent_type == CT_FOR)
+      || (chunk_is_token(pc, CT_SPAREN_OPEN) && get_chunk_parent_type(pc) == CT_FOR)
       || chunk_is_token(pc, CT_COLON)
       || chunk_is_token(pc, CT_OC_END)
       || (  chunk_is_semicolon(pc)
@@ -709,11 +788,11 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
       LOG_FMT(LTOK, "%s(%d): frm.stmt_count is %zu, frm.expr_count is %zu\n",
               __func__, __LINE__, frm.stmt_count, frm.expr_count);
    }
-
    // Mark expression starts
    LOG_FMT(LSTMT, "%s(%d): Mark expression starts: orig_line is %zu, orig_col is %zu, text() is '%s'\n",
            __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text());
    chunk_t *tmp = chunk_get_next_ncnl(pc);
+
    if (  chunk_is_token(pc, CT_ARITH)
       || chunk_is_token(pc, CT_ASSIGN)
       || chunk_is_token(pc, CT_CASE)
@@ -750,11 +829,15 @@ static void parse_cleanup(ParseFrame &frm, chunk_t *pc)
            && cpd.in_preproc != CT_PP_DEFINE)
    {
       size_t file_pp_level = ifdef_over_whole_file() ? 1 : 0;
+
       if (!cpd.unc_off_used && pc->pp_level == file_pp_level)
       {
          // fatal error
-         LOG_FMT(LERR, "%s(%d): Unmatched BRACE_CLOSE\norig_line is %zu, orig_col is %zu\n",
+         LOG_FMT(LERR, "%s(%d): Unmatched BRACE_CLOSE\n   orig_line is %zu, orig_col is %zu\n",
                  __func__, __LINE__, pc->orig_line, pc->orig_col);
+
+         log_rule_B("tok_split_gte");
+
          if (!options::tok_split_gte())
          {
             LOG_FMT(LERR, "%s(%d): Try the option 'tok_split_gte = true'\n",
@@ -790,12 +873,12 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
 
          return(true);
       }
-
       // Remove the CT_IF and close the statement
       LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
               __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-      frm.pop();
+      frm.pop(__func__, __LINE__);
       print_stack(LBCSPOP, "-IF-CCS ", frm);
+
       if (close_statement(frm, pc))
       {
          return(true);
@@ -805,6 +888,8 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
    // Check for CT_IF after CT_ELSE
    if (frm.top().stage == brace_stage_e::ELSEIF)
    {
+      log_rule_B("indent_else_if");
+
       if (  chunk_is_token(pc, CT_IF)
          && (  !options::indent_else_if()
             || !chunk_is_newline(chunk_get_prev_nc(pc))))
@@ -815,7 +900,6 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
          frm.top().stage = brace_stage_e::PAREN1;
          return(true);
       }
-
       // Jump to the 'expecting brace' stage
       frm.top().stage = brace_stage_e::BRACE2;
    }
@@ -827,6 +911,7 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
       {
          // Replace CT_TRY with CT_CATCH or CT_FINALLY on the stack & we are done
          frm.top().type = pc->type;
+
          if (language_is_set(LANG_CS))
          {
             frm.top().stage = (chunk_is_token(pc, CT_CATCH)) ? brace_stage_e::CATCH_WHEN : brace_stage_e::BRACE2;
@@ -842,11 +927,10 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
 
          return(true);
       }
-
       // Remove the CT_TRY and close the statement
       LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
               __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-      frm.pop();
+      frm.pop(__func__, __LINE__);
       print_stack(LBCSPOP, "-TRY-CCS ", frm);
 
       if (close_statement(frm, pc))
@@ -895,13 +979,12 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
 
          return(true);
       }
-
       LOG_FMT(LWARN, "%s(%d): %s, orig_line is %zu, Error: Expected 'while', got '%s'\n",
               __func__, __LINE__, cpd.filename.c_str(), pc->orig_line,
               pc->text());
       LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
               __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-      frm.pop();
+      frm.pop(__func__, __LINE__);
       print_stack(LBCSPOP, "-Error  ", frm);
       cpd.error_count++;
    }
@@ -909,10 +992,12 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
    // Insert a CT_VBRACE_OPEN, if needed
    // but not in a preprocessor
    if (  pc->type != CT_BRACE_OPEN
-      && !(pc->flags & PCF_IN_PREPROC)
+      && !pc->flags.test(PCF_IN_PREPROC)
       && (  (frm.top().stage == brace_stage_e::BRACE2)
          || (frm.top().stage == brace_stage_e::BRACE_DO)))
    {
+      log_rule_B("indent_using_block");
+
       if (  language_is_set(LANG_CS)
          && chunk_is_token(pc, CT_USING_STMT)
          && (!options::indent_using_block()))
@@ -932,7 +1017,7 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
                  __func__, __LINE__, frm.brace_level);
          log_pcf_flags(LBCSPOP, pc->flags);
 
-         frm.push(*vbrace, brace_stage_e::NONE);
+         frm.push(vbrace, __func__, __LINE__, brace_stage_e::NONE);
          // "+VBrace");
 
          frm.top().parent = parent;
@@ -946,7 +1031,7 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
          frm.expr_count = 0;
          LOG_FMT(LTOK, "%s(%d): frm.stmt_count is %zu, frm.expr_count is %zu\n",
                  __func__, __LINE__, frm.stmt_count, frm.expr_count);
-         pc->flags     |= PCF_STMT_START | PCF_EXPR_START;
+         chunk_flags_set(pc, PCF_STMT_START | PCF_EXPR_START);
          frm.stmt_count = 1;
          frm.expr_count = 1;
          LOG_FMT(LSTMT, "%s(%d): orig_line is %zu, 2.marked '%s' as stmt start\n",
@@ -975,11 +1060,10 @@ static bool check_complex_statements(ParseFrame &frm, chunk_t *pc)
       // Throw out the complex statement
       LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
               __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-      frm.pop();
+      frm.pop(__func__, __LINE__);
       print_stack(LBCSPOP, "-Error  ", frm);
       cpd.error_count++;
    }
-
    return(false);
 } // check_complex_statements
 
@@ -997,7 +1081,6 @@ static bool handle_complex_close(ParseFrame &frm, chunk_t *pc)
 
          return(true);
       }
-
       // PAREN1 always => BRACE2
       frm.top().stage = brace_stage_e::BRACE2;
    }
@@ -1011,11 +1094,12 @@ static bool handle_complex_close(ParseFrame &frm, chunk_t *pc)
 
          // If the next chunk isn't CT_ELSE, close the statement
          chunk_t *next = chunk_get_next_ncnl(pc);
+
          if (next != nullptr && next->type != CT_ELSE)
          {
             LOG_FMT(LBCSPOP, "%s(%d): no CT_ELSE, pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
                     __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-            frm.pop();
+            frm.pop(__func__, __LINE__);
             print_stack(LBCSPOP, "-IF-HCS ", frm);
 
             return(close_statement(frm, pc));
@@ -1028,13 +1112,14 @@ static bool handle_complex_close(ParseFrame &frm, chunk_t *pc)
 
          // If the next chunk isn't CT_CATCH or CT_FINALLY, close the statement
          chunk_t *next = chunk_get_next_ncnl(pc);
+
          if (  next != nullptr
             && next->type != CT_CATCH
             && next->type != CT_FINALLY)
          {
             LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
                     __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-            frm.pop();
+            frm.pop(__func__, __LINE__);
             print_stack(LBCSPOP, "-TRY-HCS ", frm);
 
             return(close_statement(frm, pc));
@@ -1046,7 +1131,7 @@ static bool handle_complex_close(ParseFrame &frm, chunk_t *pc)
                  __func__, __LINE__, get_token_name(frm.top().type));
          LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
                  __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-         frm.pop();
+         frm.pop(__func__, __LINE__);
          print_stack(LBCSPOP, "-HCC B2 ", frm);
 
          return(close_statement(frm, pc));
@@ -1069,7 +1154,7 @@ static bool handle_complex_close(ParseFrame &frm, chunk_t *pc)
               __func__, __LINE__, get_token_name(frm.top().type));
       LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
               __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-      frm.pop();
+      frm.pop(__func__, __LINE__);
       print_stack(LBCSPOP, "-HCC WoDS ", frm);
 
       return(close_statement(frm, pc));
@@ -1095,16 +1180,18 @@ static void mark_namespace(chunk_t *pns)
    bool    is_using = false;
 
    chunk_t *pc = chunk_get_prev_ncnl(pns);
+
    if (chunk_is_token(pc, CT_USING))
    {
       is_using = true;
       set_chunk_parent(pns, CT_USING);
    }
-
    pc = chunk_get_next_ncnl(pns);
+
    while (pc != nullptr)
    {
       set_chunk_parent(pc, CT_NAMESPACE);
+
       if (pc->type != CT_BRACE_OPEN)
       {
          if (chunk_is_token(pc, CT_SEMICOLON))
@@ -1118,15 +1205,23 @@ static void mark_namespace(chunk_t *pns)
          pc = chunk_get_next_ncnl(pc);
          continue;
       }
+      log_rule_B("indent_namespace_limit");
 
       if (  (options::indent_namespace_limit() > 0)
          && ((br_close = chunk_skip_to_match(pc)) != nullptr))
       {
          // br_close->orig_line is always >= pc->orig_line;
-         size_t diff = br_close->orig_line - pc->orig_line;
+         size_t numberOfLines = br_close->orig_line - pc->orig_line - 1;                 // Issue #2345
+         LOG_FMT(LTOK, "%s(%d): br_close->orig_line is %zu, pc->orig_line is %zu\n",
+                 __func__, __LINE__, br_close->orig_line, pc->orig_line);
+         LOG_FMT(LTOK, "%s(%d): numberOfLines is %zu, indent_namespace_limit() is %d\n",
+                 __func__, __LINE__, numberOfLines, options::indent_namespace_limit());
 
-         if (diff > options::indent_namespace_limit())
+         log_rule_B("indent_namespace_limit");
+
+         if (numberOfLines > options::indent_namespace_limit())
          {
+            LOG_FMT(LTOK, "%s(%d): PCF_LONG_BLOCK is set\n", __func__, __LINE__);
             chunk_flags_set(pc, PCF_LONG_BLOCK);
             chunk_flags_set(br_close, PCF_LONG_BLOCK);
          }
@@ -1142,8 +1237,9 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after, const ParseFrame &frm)
    LOG_FUNC_ENTRY();
 
    chunk_t chunk;
+
+   set_chunk_parent(&chunk, frm.top().type);
    chunk.orig_line   = pc->orig_line;
-   chunk.parent_type = frm.top().type;
    chunk.level       = frm.level;
    chunk.brace_level = frm.brace_level;
    chunk.flags       = pc->flags & PCF_COPY_FLAGS;
@@ -1152,18 +1248,17 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after, const ParseFrame &frm)
    if (after)
    {
       chunk.orig_col = pc->orig_col;
-      chunk.type     = CT_VBRACE_CLOSE;
+      set_chunk_type(&chunk, CT_VBRACE_CLOSE);
       return(chunk_add_after(&chunk, pc));
    }
-
-
    chunk_t *ref = chunk_get_prev(pc);
+
    if (ref == nullptr)
    {
       return(nullptr);
    }
 
-   if ((ref->flags & PCF_IN_PREPROC) == 0)
+   if (!ref->flags.test(PCF_IN_PREPROC))
    {
       chunk.flags &= ~PCF_IN_PREPROC;
    }
@@ -1174,18 +1269,19 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after, const ParseFrame &frm)
       ref->brace_level++;
       ref = chunk_get_prev(ref);
    }
+
    if (ref == nullptr)
    {
       return(nullptr);
    }
 
    // Don't back into a preprocessor
-   if (  (pc->flags & PCF_IN_PREPROC) == 0
-      && (ref->flags & PCF_IN_PREPROC) != 0)
+   if (  !pc->flags.test(PCF_IN_PREPROC)
+      && ref->flags.test(PCF_IN_PREPROC))
    {
       if (chunk_is_token(ref, CT_PREPROC_BODY))
       {
-         while (ref != nullptr && (ref->flags & PCF_IN_PREPROC))
+         while (ref != nullptr && ref->flags.test(PCF_IN_PREPROC))
          {
             ref = chunk_get_prev(ref);
          }
@@ -1195,15 +1291,15 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after, const ParseFrame &frm)
          ref = chunk_get_next(ref);
       }
    }
+
    if (ref == nullptr)
    {
       return(nullptr);
    }
-
    chunk.orig_line = ref->orig_line;
    chunk.orig_col  = ref->orig_col;
    chunk.column    = ref->column + ref->len() + 1;
-   chunk.type      = CT_VBRACE_OPEN;
+   set_chunk_type(&chunk, CT_VBRACE_OPEN);
 
    return(chunk_add_after(&chunk, ref));
 } // insert_vbrace
@@ -1212,6 +1308,7 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after, const ParseFrame &frm)
 bool close_statement(ParseFrame &frm, chunk_t *pc)
 {
    LOG_FUNC_ENTRY();
+
    if (pc == nullptr)
    {
       throw invalid_argument(string(__func__) + ":" + to_string(__LINE__)
@@ -1230,12 +1327,12 @@ bool close_statement(ParseFrame &frm, chunk_t *pc)
       LOG_FMT(LSTMT, "%s(%d): orig_line is %zu> reset2 stmt on '%s'\n",
               __func__, __LINE__, pc->orig_line, pc->text());
    }
-
    /*
     * Insert a CT_VBRACE_CLOSE, if needed:
     * If we are in a virtual brace and we are not ON a CT_VBRACE_CLOSE add one
     */
    chunk_t *vbc = pc;
+
    if (frm.top().type == CT_VBRACE_OPEN)
    {
       // If the current token has already been consumed, then add after it
@@ -1257,7 +1354,7 @@ bool close_statement(ParseFrame &frm, chunk_t *pc)
          log_pcf_flags(LBCSPOP, pc->flags);
          LOG_FMT(LBCSPOP, "%s(%d): pc->orig_line is %zu, orig_col is %zu, text() is '%s', type is %s\n",
                  __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text(), get_token_name(pc->type));
-         frm.pop();
+         frm.pop(__func__, __LINE__);
 
          // Update the token level
          pc->level       = frm.level;
