@@ -5,22 +5,27 @@
  * @author  Ben Gardner
  * @license GPL v2+
  */
+
 #include "braces.h"
-#include "uncrustify_types.h"
+
 #include "chunk_list.h"
+#include "combine.h"
+#include "combine_tools.h"
+#include "error_types.h"
+#include "language_tools.h"
+#include "log_rules.h"
+#include "newlines.h"
 #include "prototypes.h"
 #include "unc_ctype.h"
 #include "uncrustify.h"
-#include "combine.h"
-#include "newlines.h"
-#include "chunk_list.h"
-#include "language_tools.h"
+#include "uncrustify_types.h"
 
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <vector>
 
+
 using namespace uncrustify;
 
 using std::vector;
@@ -120,13 +125,13 @@ static bool paren_multiline_before_brace(chunk_t *brace)
 {
    if (  brace == nullptr
       || (brace->type != CT_BRACE_OPEN && brace->type != CT_BRACE_CLOSE)
-      || (  brace->parent_type != CT_IF
-         && brace->parent_type != CT_ELSEIF
-         && brace->parent_type != CT_FOR
-         && brace->parent_type != CT_USING_STMT
-         && brace->parent_type != CT_WHILE
-         && brace->parent_type != CT_FUNC_CLASS_DEF
-         && brace->parent_type != CT_FUNC_DEF))
+      || (  get_chunk_parent_type(brace) != CT_IF
+         && get_chunk_parent_type(brace) != CT_ELSEIF
+         && get_chunk_parent_type(brace) != CT_FOR
+         && get_chunk_parent_type(brace) != CT_USING_STMT
+         && get_chunk_parent_type(brace) != CT_WHILE
+         && get_chunk_parent_type(brace) != CT_FUNC_CLASS_DEF
+         && get_chunk_parent_type(brace) != CT_FUNC_DEF))
    {
       return(false);
    }
@@ -143,16 +148,15 @@ static bool paren_multiline_before_brace(chunk_t *brace)
    {
       return(false);
    }
-
    // determine number of lines in the parenthesis pair spans
-   auto       nl_count = size_t {};
+   auto       nl_count = size_t{};
    const auto ret_flag = newlines_between(paren_open, paren_close, nl_count);
+
    if (!ret_flag)
    {
       LOG_FMT(LERR, "%s(%d): newlines_between error\n", __func__, __LINE__);
       return(false);
    }
-
    // nl_count = 0 -> 1 line
    return(nl_count > 0);
 }
@@ -161,34 +165,10 @@ static bool paren_multiline_before_brace(chunk_t *brace)
 void do_braces(void)
 {
    LOG_FUNC_ENTRY();
-   if (  options::mod_full_brace_if_chain()
-      || options::mod_full_brace_if_chain_only())
-   {
-      mod_full_brace_if_chain();
-   }
-
-   if ((options::mod_full_brace_if() |
-        options::mod_full_brace_do() |
-        options::mod_full_brace_for() |
-        options::mod_full_brace_using() |
-        options::mod_full_brace_while()) & IARF_REMOVE)
-   {
-      examine_braces();
-   }
-
-   // convert vbraces if needed
-   if ((options::mod_full_brace_if() |
-        options::mod_full_brace_do() |
-        options::mod_full_brace_for() |
-        options::mod_full_brace_function() |
-        options::mod_full_brace_using() |
-        options::mod_full_brace_while()) & IARF_ADD)
-   {
-      convert_vbrace_to_brace();
-   }
-
    // Mark one-liners
+   // Issue #2232 put this at the beginning
    chunk_t *pc = chunk_get_head();
+
    while ((pc = chunk_get_next_ncnl(pc)) != nullptr)
    {
       if (pc->type != CT_BRACE_OPEN && pc->type != CT_VBRACE_OPEN)
@@ -199,20 +179,22 @@ void do_braces(void)
       const c_token_t brc_type = c_token_t(pc->type + 1); // corresponds to closing type
       // Detect empty bodies
       chunk_t         *tmp = chunk_get_next_ncnl(pc);
+
       if (chunk_is_token(tmp, brc_type))
       {
          chunk_flags_set(br_open, PCF_EMPTY_BODY);
          chunk_flags_set(tmp, PCF_EMPTY_BODY);
       }
-
       // Scan for the brace close or a newline
       tmp = br_open;
+
       while ((tmp = chunk_get_next_nc(tmp)) != nullptr)
       {
          if (chunk_is_newline(tmp))
          {
             break;
          }
+
          if (tmp->type == brc_type && br_open->level == tmp->level)
          {
             flag_series(br_open, tmp, PCF_ONE_LINER);
@@ -220,11 +202,53 @@ void do_braces(void)
          }
       }
    }
+   log_rule_B("mod_full_brace_if_chain");
+   log_rule_B("mod_full_brace_if_chain_only");
+
+   if (  options::mod_full_brace_if_chain()
+      || options::mod_full_brace_if_chain_only())
+   {
+      mod_full_brace_if_chain();
+   }
+   log_rule_B("mod_full_brace_if");
+   log_rule_B("mod_full_brace_do");
+   log_rule_B("mod_full_brace_for");
+   log_rule_B("mod_full_brace_using");
+   log_rule_B("mod_full_brace_while");
+
+   if ((options::mod_full_brace_if() |
+        options::mod_full_brace_do() |
+        options::mod_full_brace_for() |
+        options::mod_full_brace_using() |
+        options::mod_full_brace_while()) & IARF_REMOVE)
+   {
+      examine_braces();
+   }
+   // convert vbraces if needed
+   log_rule_B("mod_full_brace_if");
+   log_rule_B("mod_full_brace_do");
+   log_rule_B("mod_full_brace_for");
+   log_rule_B("mod_full_brace_function");
+   log_rule_B("mod_full_brace_using");
+   log_rule_B("mod_full_brace_while");
+
+   if ((options::mod_full_brace_if() |
+        options::mod_full_brace_do() |
+        options::mod_full_brace_for() |
+        options::mod_full_brace_function() |
+        options::mod_full_brace_using() |
+        options::mod_full_brace_while()) & IARF_ADD)
+   {
+      convert_vbrace_to_brace();
+   }
+   log_rule_B("mod_case_brace");
 
    if (options::mod_case_brace() != IARF_IGNORE)
    {
       mod_case_brace();
    }
+   log_rule_B("mod_move_case_break");
+
    if (options::mod_move_case_break())
    {
       move_case_break();
@@ -236,25 +260,32 @@ static void examine_braces(void)
 {
    LOG_FUNC_ENTRY();
 
+   log_rule_B("mod_full_brace_nl_block_rem_mlcond");
    const auto multiline_block = options::mod_full_brace_nl_block_rem_mlcond();
 
+   log_rule_B("mod_full_brace_if");
+   log_rule_B("mod_full_brace_do");
+   log_rule_B("mod_full_brace_for");
+   log_rule_B("mod_full_brace_using");
+   log_rule_B("mod_full_brace_while");
+
    for (auto pc = chunk_get_tail(); pc != nullptr;)
    {
       auto prev = chunk_get_prev_type(pc, CT_BRACE_OPEN, -1);
 
       if (  chunk_is_token(pc, CT_BRACE_OPEN)
-         && ((pc->flags & PCF_IN_PREPROC) == 0)
-         && (  (  (  pc->parent_type == CT_IF
-                  || pc->parent_type == CT_ELSE
-                  || pc->parent_type == CT_ELSEIF)
+         && !pc->flags.test(PCF_IN_PREPROC)
+         && (  (  (  get_chunk_parent_type(pc) == CT_IF
+                  || get_chunk_parent_type(pc) == CT_ELSE
+                  || get_chunk_parent_type(pc) == CT_ELSEIF)
                && options::mod_full_brace_if() == IARF_REMOVE)
-            || (  pc->parent_type == CT_DO
+            || (  get_chunk_parent_type(pc) == CT_DO
                && options::mod_full_brace_do() == IARF_REMOVE)
-            || (  pc->parent_type == CT_FOR
+            || (  get_chunk_parent_type(pc) == CT_FOR
                && options::mod_full_brace_for() == IARF_REMOVE)
-            || (  pc->parent_type == CT_USING_STMT
+            || (  get_chunk_parent_type(pc) == CT_USING_STMT
                && options::mod_full_brace_using() == IARF_REMOVE)
-            || (  pc->parent_type == CT_WHILE
+            || (  get_chunk_parent_type(pc) == CT_WHILE
                && options::mod_full_brace_while() == IARF_REMOVE)))
       {
          if (multiline_block && paren_multiline_before_brace(pc))
@@ -262,19 +293,19 @@ static void examine_braces(void)
             pc = prev;
             continue;
          }
-
          examine_brace(pc);
       }
-
       pc = prev;
    }
-}
+} // examine_braces
 
 
 static bool should_add_braces(chunk_t *vbopen)
 {
    LOG_FUNC_ENTRY();
+   log_rule_B("mod_full_brace_nl");
    const size_t nl_max = options::mod_full_brace_nl();
+
    if (nl_max == 0)
    {
       return(false);
@@ -285,6 +316,7 @@ static bool should_add_braces(chunk_t *vbopen)
    size_t  nl_count = 0;
 
    chunk_t *pc = nullptr;
+
    for (pc = chunk_get_next_nc(vbopen, scope_e::PREPROC);
         pc != nullptr && pc->level > vbopen->level;
         pc = chunk_get_next_nc(pc, scope_e::PREPROC))
@@ -314,18 +346,20 @@ static bool can_remove_braces(chunk_t *bopen)
            __func__, __LINE__, bopen->orig_line);
 
    // Cannot remove braces inside a preprocessor
-   if (bopen->flags & PCF_IN_PREPROC)
+   if (bopen->flags.test(PCF_IN_PREPROC))
    {
       return(false);
    }
    chunk_t *pc = chunk_get_next_ncnl(bopen, scope_e::PREPROC);
+
    if (chunk_is_token(pc, CT_BRACE_CLOSE))
    {
       // Can't remove empty statement
       return(false);
    }
+   const size_t level = bopen->level + 1;
 
-   const size_t level  = bopen->level + 1;
+   log_rule_B("mod_full_brace_nl");
    const size_t nl_max = options::mod_full_brace_nl();
    chunk_t      *prev  = nullptr;
 
@@ -338,11 +372,13 @@ static bool can_remove_braces(chunk_t *bopen)
    pc = chunk_get_next_nc(bopen, scope_e::ALL);
    LOG_FMT(LBRDEL, "%s(%d):  - begin with token '%s', orig_line is %zu, orig_col is %zu\n",
            __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
+
    while (pc != nullptr && pc->level >= level)
    {
       LOG_FMT(LBRDEL, "%s(%d): test token '%s', orig_line is %zu, orig_col is %zu\n",
               __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
-      if (pc->flags & PCF_IN_PREPROC)
+
+      if (pc->flags.test(PCF_IN_PREPROC))
       {
          // Cannot remove braces that contain a preprocessor
          return(false);
@@ -351,6 +387,7 @@ static bool can_remove_braces(chunk_t *bopen)
       if (chunk_is_newline(pc))
       {
          nl_count += pc->nl_count;
+
          if (nl_max > 0 && nl_count > nl_max)
          {
             LOG_FMT(LBRDEL, "%s(%d):  exceeded %zu newlines\n",
@@ -366,7 +403,15 @@ static bool can_remove_braces(chunk_t *bopen)
          }
          else if (chunk_is_token(pc, CT_BRACE_CLOSE))
          {
+            if (br_count == 0)
+            {
+               fprintf(stderr, "%s(%d): br_count is ZERO, cannot be decremented, at line %zu, column %zu\n",
+                       __func__, __LINE__, pc->orig_line, pc->orig_col);
+               log_flush(true);
+               exit(EX_SOFTWARE);
+            }
             br_count--;
+
             if (pc->level == level)
             {
                // mean a statement in a braces { stmt; }
@@ -389,7 +434,6 @@ static bool can_remove_braces(chunk_t *bopen)
                LOG_FMT(LBRDEL, "%s(%d):  no close brace\n", __func__, __LINE__);
                return(false);
             }
-
             LOG_FMT(LBRDEL, "%s(%d): text() '%s', orig_line is %zu, semi_count is %zu\n",
                     __func__, __LINE__, pc->text(), pc->orig_line, semi_count);
 
@@ -411,6 +455,7 @@ static bool can_remove_braces(chunk_t *bopen)
                   && chunk_is_token(prev, CT_FPAREN_CLOSE)))
             {
                hit_semi |= chunk_is_semicolon(pc);
+
                if (++semi_count > 1)
                {
                   LOG_FMT(LBRDEL, "%s(%d):  bailed on %zu because of '%s' on line %zu\n",
@@ -430,22 +475,21 @@ static bool can_remove_braces(chunk_t *bopen)
       return(false);
    }
 
-   if (chunk_is_token(pc, CT_BRACE_CLOSE) && pc->parent_type == CT_IF)
+   if (chunk_is_token(pc, CT_BRACE_CLOSE) && get_chunk_parent_type(pc) == CT_IF)
    {
       chunk_t *next     = chunk_get_next_ncnl(pc, scope_e::PREPROC);
       chunk_t *tmp_prev = chunk_get_prev_ncnl(pc, scope_e::PREPROC);
 
       if (  chunk_is_token(next, CT_ELSE)
          && (chunk_is_token(tmp_prev, CT_BRACE_CLOSE) || chunk_is_token(tmp_prev, CT_VBRACE_CLOSE))
-         && tmp_prev->parent_type == CT_IF)
+         && get_chunk_parent_type(tmp_prev) == CT_IF)
       {
          LOG_FMT(LBRDEL, "%s(%d):  - bailed on '%s'[%s] on line %zu due to 'if' and 'else' sequence\n",
-                 __func__, __LINE__, get_token_name(pc->type), get_token_name(pc->parent_type),
+                 __func__, __LINE__, get_token_name(pc->type), get_token_name(get_chunk_parent_type(pc)),
                  pc->orig_line);
          return(false);
       }
    }
-
    LOG_FMT(LBRDEL, "%s(%d):  - end on '%s' on line %zu. if_count is %zu semi_count is %zu\n",
            __func__, __LINE__, get_token_name(pc->type), pc->orig_line, if_count, semi_count);
 
@@ -459,7 +503,9 @@ static void examine_brace(chunk_t *bopen)
    LOG_FMT(LBRDEL, "%s(%d): start on orig_line %zu, bopen->level is %zu\n",
            __func__, __LINE__, bopen->orig_line, bopen->level);
 
-   const size_t level  = bopen->level + 1;
+   const size_t level = bopen->level + 1;
+
+   log_rule_B("mod_full_brace_nl");
    const size_t nl_max = options::mod_full_brace_nl();
 
    chunk_t      *prev      = nullptr;
@@ -470,6 +516,7 @@ static void examine_brace(chunk_t *bopen)
    int          br_count   = 0;
 
    chunk_t      *pc = chunk_get_next_nc(bopen);
+
    while (pc != nullptr && pc->level >= level)
    {
       if (chunk_is_token(pc, CT_NEWLINE))
@@ -482,7 +529,8 @@ static void examine_brace(chunk_t *bopen)
          LOG_FMT(LBRDEL, "%s(%d): orig_line is %zu, orig_col is %zu, text() '%s'\n",
                  __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text());
       }
-      if (pc->flags & PCF_IN_PREPROC)
+
+      if (pc->flags.test(PCF_IN_PREPROC))
       {
          // Cannot remove braces that contain a preprocessor
          LOG_FMT(LBRDEL, "%s(%d):  PREPROC\n", __func__, __LINE__);
@@ -492,6 +540,7 @@ static void examine_brace(chunk_t *bopen)
       if (chunk_is_newline(pc))
       {
          nl_count += pc->nl_count;
+
          if (nl_max > 0 && nl_count > nl_max)
          {
             LOG_FMT(LBRDEL, "%s(%d):  exceeded %zu newlines\n",
@@ -503,6 +552,7 @@ static void examine_brace(chunk_t *bopen)
       {
          LOG_FMT(LBRDEL, "%s(%d): for pc->text() '%s', pc->level is %zu,  bopen->level is %zu\n",
                  __func__, __LINE__, pc->text(), pc->level, bopen->level);
+
          if (  chunk_is_token(pc, CT_BRACE_OPEN)
             && pc->level == bopen->level)
          {
@@ -513,12 +563,21 @@ static void examine_brace(chunk_t *bopen)
          else if (  chunk_is_token(pc, CT_BRACE_CLOSE)
                  && pc->level == bopen->level)
          {
+            if (br_count == 0)
+            {
+               fprintf(stderr, "%s(%d): br_count is ZERO, cannot be decremented, at line %zu, column %zu\n",
+                       __func__, __LINE__, pc->orig_line, pc->orig_col);
+               log_flush(true);
+               exit(EX_SOFTWARE);
+            }
             br_count--;
             LOG_FMT(LBRDEL, "%s(%d): br_count is now %d, pc->level is %zu,  bopen->level is %zu\n",
                     __func__, __LINE__, br_count, pc->level, bopen->level);
+
             if (br_count == 0)
             {
                chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
+
                if (next == nullptr || next->type != CT_BRACE_CLOSE)
                {
                   LOG_FMT(LBRDEL, "%s(%d):  junk after close brace\n", __func__, __LINE__);
@@ -531,9 +590,9 @@ static void examine_brace(chunk_t *bopen)
          {
             if_count++;
          }
-
          LOG_FMT(LBRDEL, "%s(%d): pc->level is %zu, level is %zu\n",
                  __func__, __LINE__, pc->level, level);
+
          if (pc->level == level)
          {
             if (semi_count > 0 && hit_semi)
@@ -542,7 +601,6 @@ static void examine_brace(chunk_t *bopen)
                LOG_FMT(LBRDEL, "%s(%d):  no close brace\n", __func__, __LINE__);
                return;
             }
-
             LOG_FMT(LBRDEL, "%s(%d): text() '%s', orig_line is %zu, semi_count is %zu\n",
                     __func__, __LINE__, pc->text(), pc->orig_line, semi_count);
 
@@ -565,6 +623,7 @@ static void examine_brace(chunk_t *bopen)
             }
             LOG_FMT(LBRDEL, "%s(%d): for pc->text() '%s', pc->level is %zu,  bopen->level is %zu\n",
                     __func__, __LINE__, pc->text(), pc->level, bopen->level);
+
             if (  chunk_is_semicolon(pc)
                || chunk_is_token(pc, CT_IF)
                || chunk_is_token(pc, CT_ELSEIF)
@@ -582,6 +641,7 @@ static void examine_brace(chunk_t *bopen)
                semi_count++;
                LOG_FMT(LBRDEL, "%s(%d): semi_count is %zu\n",
                        __func__, __LINE__, semi_count);
+
                if (semi_count > 1)
                {
                   LOG_FMT(LBRDEL, "%s(%d):  bailed on %zu because of '%s' on line %zu\n",
@@ -592,7 +652,7 @@ static void examine_brace(chunk_t *bopen)
          }
       }
       prev = pc;
-      pc   = chunk_get_next_nc(pc);
+      pc   = chunk_get_next(pc);                  // Issue #1907
    }
 
    if (pc == nullptr)
@@ -600,13 +660,13 @@ static void examine_brace(chunk_t *bopen)
       LOG_FMT(LBRDEL, "%s(%d): pc is nullptr\n", __func__, __LINE__);
       return;
    }
-
    LOG_FMT(LBRDEL, "%s(%d):  - end on '%s' on line %zu. if_count is %zu, semi_count is %zu\n",
            __func__, __LINE__, get_token_name(pc->type), pc->orig_line, if_count, semi_count);
 
    if (chunk_is_token(pc, CT_BRACE_CLOSE))
    {
       chunk_t *next = chunk_get_next_ncnl(pc);
+
       if (next != nullptr)
       {
          while (chunk_is_token(next, CT_VBRACE_CLOSE))
@@ -619,6 +679,7 @@ static void examine_brace(chunk_t *bopen)
             LOG_FMT(LBRDEL, "%s(%d): orig_line is %zu, orig_col is %zu, next is '%s'\n",
                     __func__, __LINE__, next->orig_line, next->orig_col, get_token_name(next->type));
          }
+
          if (  if_count > 0
             && (chunk_is_token(next, CT_ELSE) || chunk_is_token(next, CT_ELSEIF)))
          {
@@ -627,16 +688,18 @@ static void examine_brace(chunk_t *bopen)
             return;
          }
       }
-
       LOG_FMT(LBRDEL, "%s(%d): semi_count is %zu\n",
               __func__, __LINE__, semi_count);
+
       if (semi_count > 0)
       {
          LOG_FMT(LBRDEL, "%s(%d): bopen->parent_type is %s\n",
-                 __func__, __LINE__, get_token_name(bopen->parent_type));
-         if (bopen->parent_type == CT_ELSE)
+                 __func__, __LINE__, get_token_name(get_chunk_parent_type(bopen)));
+
+         if (get_chunk_parent_type(bopen) == CT_ELSE)
          {
             chunk_t *tmp_next = chunk_get_next_ncnl(bopen);
+
             if (chunk_is_token(tmp_next, CT_IF))
             {
                chunk_t *tmp_prev = chunk_get_prev_ncnl(bopen);
@@ -646,6 +709,9 @@ static void examine_brace(chunk_t *bopen)
                chunk_del(bopen);
                chunk_del(pc);
                newline_del_between(tmp_prev, tmp_next);
+
+               log_rule_B("nl_else_if");
+
                if (options::nl_else_if() & IARF_ADD)
                {
                   newline_add_between(tmp_prev, tmp_next);
@@ -653,7 +719,6 @@ static void examine_brace(chunk_t *bopen)
                return;
             }
          }
-
          // we have a pair of braces with only 1 statement inside
          LOG_FMT(LBRDEL, "%s(%d): we have a pair of braces with only 1 statement inside\n",
                  __func__, __LINE__);
@@ -678,45 +743,65 @@ static void examine_brace(chunk_t *bopen)
 static void convert_brace(chunk_t *br)
 {
    LOG_FUNC_ENTRY();
+
    if (  br == nullptr
-      || (br->flags & PCF_KEEP_BRACE))
+      || br->flags.test(PCF_KEEP_BRACE))
    {
       return;
    }
-
    chunk_t *tmp;
+
    if (chunk_is_token(br, CT_BRACE_OPEN))
    {
       set_chunk_type(br, CT_VBRACE_OPEN);
       br->str.clear();
       tmp = chunk_get_prev(br);
+
+      if (tmp == nullptr)
+      {
+         return;
+      }
    }
    else if (chunk_is_token(br, CT_BRACE_CLOSE))
    {
       set_chunk_type(br, CT_VBRACE_CLOSE);
       br->str.clear();
       tmp = chunk_get_next(br);
+
+      if (tmp == nullptr)
+      {
+         return;
+      }
    }
    else
    {
       return;
    }
-   LOG_FMT(LGUY, "%s(%d): br->type is %s, br->parent_type is %s\n",
-           __func__, __LINE__, get_token_name(br->type), get_token_name(br->parent_type));
-   LOG_FMT(LGUY, "%s(%d): br->orig_line is %zu, br->orig_col is %zu\n",
-           __func__, __LINE__, br->orig_line, br->orig_col);
 
    if (chunk_is_newline(tmp))
    {
       if (tmp->nl_count > 1)
       {
-         tmp->nl_count--;
+         if (!br->flags.test(PCF_ONE_LINER)) // Issue #2232
+         {
+            if (tmp->nl_count == 0)
+            {
+               fprintf(stderr, "%s(%d): tmp->nl_count is ZERO, cannot be decremented, at line %zu, column %zu\n",
+                       __func__, __LINE__, tmp->orig_line, tmp->orig_col);
+               log_flush(true);
+               exit(EX_SOFTWARE);
+            }
+            tmp->nl_count--;
+            LOG_FMT(LBRDEL, "%s(%d): tmp->nl_count is %zu\n",
+                    __func__, __LINE__, tmp->nl_count);
+         }
       }
       else
       {
          // Issue #2219
          // look for opening brace
          chunk_t *brace;
+
          if (chunk_is_token(br, CT_VBRACE_OPEN))
          {
             brace = tmp;
@@ -724,11 +809,13 @@ static void convert_brace(chunk_t *br)
          else if (chunk_is_token(br, CT_VBRACE_CLOSE))
          {
             brace = chunk_skip_to_match_rev(br);
+
             if (brace == nullptr)
             {
                brace = chunk_get_prev_type(br, CT_BRACE_OPEN, br->level);
             }
          }
+
          if (  chunk_is_token(br, CT_VBRACE_OPEN)
             || (  chunk_is_token(br, CT_VBRACE_CLOSE)
                && brace->orig_line < tmp->orig_line))
@@ -746,6 +833,7 @@ static void convert_brace(chunk_t *br)
 static void convert_vbrace(chunk_t *vbr)
 {
    LOG_FUNC_ENTRY();
+
    if (vbr == nullptr)
    {
       return;
@@ -761,6 +849,7 @@ static void convert_vbrace(chunk_t *vbr)
        * preprocessor.
        */
       chunk_t *tmp = chunk_get_next(vbr);
+
       if (chunk_is_token(tmp, CT_PREPROC))
       {
          tmp = chunk_get_next(vbr, scope_e::PREPROC);
@@ -779,9 +868,11 @@ static void convert_vbrace(chunk_t *vbr)
        * the close brace.
        */
       chunk_t *tmp = chunk_get_next(vbr);
+
       if (chunk_is_comment(tmp))
       {
          tmp = chunk_get_next(tmp);
+
          if (chunk_is_newline(tmp))
          {
             chunk_move_after(vbr, tmp);
@@ -789,7 +880,7 @@ static void convert_vbrace(chunk_t *vbr)
          }
       }
    }
-}
+} // convert_vbrace
 
 
 static void convert_vbrace_to_brace(void)
@@ -797,55 +888,64 @@ static void convert_vbrace_to_brace(void)
    LOG_FUNC_ENTRY();
 
    // Find every vbrace open
+   log_rule_B("mod_full_brace_if");
+   log_rule_B("mod_full_brace_if_chain");
+   log_rule_B("mod_full_brace_for");
+   log_rule_B("mod_full_brace_do");
+   log_rule_B("mod_full_brace_while");
+   log_rule_B("mod_full_brace_using");
+   log_rule_B("mod_full_brace_function");
+
    for (chunk_t *pc = chunk_get_head(); pc != nullptr; pc = chunk_get_next_ncnl(pc))
    {
       if (pc->type != CT_VBRACE_OPEN)
       {
          continue;
       }
+      auto const in_preproc = pc->flags.test(PCF_IN_PREPROC);
 
-      bool in_preproc = (pc->flags & PCF_IN_PREPROC);
-
-      if (  (  (  pc->parent_type == CT_IF
-               || pc->parent_type == CT_ELSE
-               || pc->parent_type == CT_ELSEIF)
+      if (  (  (  get_chunk_parent_type(pc) == CT_IF
+               || get_chunk_parent_type(pc) == CT_ELSE
+               || get_chunk_parent_type(pc) == CT_ELSEIF)
             && (options::mod_full_brace_if() & IARF_ADD)
             && !options::mod_full_brace_if_chain())
-         || (  pc->parent_type == CT_FOR
+         || (  get_chunk_parent_type(pc) == CT_FOR
             && (options::mod_full_brace_for() & IARF_ADD))
-         || (  pc->parent_type == CT_DO
+         || (  get_chunk_parent_type(pc) == CT_DO
             && (options::mod_full_brace_do() & IARF_ADD))
-         || (  pc->parent_type == CT_WHILE
+         || (  get_chunk_parent_type(pc) == CT_WHILE
             && (options::mod_full_brace_while() & IARF_ADD))
-         || (  pc->parent_type == CT_USING_STMT
+         || (  get_chunk_parent_type(pc) == CT_USING_STMT
             && (options::mod_full_brace_using() & IARF_ADD))
-         || (  pc->parent_type == CT_FUNC_DEF
+         || (  get_chunk_parent_type(pc) == CT_FUNC_DEF
             && (options::mod_full_brace_function() & IARF_ADD)))
       {
          // Find the matching vbrace close
          chunk_t *vbc = nullptr;
          chunk_t *tmp = pc;
+
          while ((tmp = chunk_get_next(tmp)) != nullptr)
          {
-            if (in_preproc && ((tmp->flags & PCF_IN_PREPROC) == 0))
+            if (in_preproc && !tmp->flags.test(PCF_IN_PREPROC))
             {
                // Can't leave a preprocessor
                break;
             }
+
             if (  pc->brace_level == tmp->brace_level
                && chunk_is_token(tmp, CT_VBRACE_CLOSE)
-               && pc->parent_type == tmp->parent_type
+               && get_chunk_parent_type(pc) == get_chunk_parent_type(tmp)
                && ((tmp->flags & PCF_IN_PREPROC) == (pc->flags & PCF_IN_PREPROC)))
             {
                vbc = tmp;
                break;
             }
          }
+
          if (vbc == nullptr)
          {
             continue;
          }
-
          // if we found a corresponding virtual closing brace
          convert_vbrace(pc);   // convert both the opening
          convert_vbrace(vbc);  // and closing brace
@@ -860,11 +960,13 @@ chunk_t *insert_comment_after(chunk_t *ref, c_token_t cmt_type,
    LOG_FUNC_ENTRY();
 
    chunk_t new_cmt = *ref;
+
    new_cmt.prev  = nullptr;
    new_cmt.next  = nullptr;
    new_cmt.flags = (ref->flags & PCF_COPY_FLAGS);
-   new_cmt.type  = cmt_type;
+   set_chunk_type(&new_cmt, cmt_type);
    new_cmt.str.clear();
+
    if (cmt_type == CT_COMMENT_CPP)
    {
       new_cmt.str.append("// ");
@@ -912,7 +1014,6 @@ static void append_tag_name(unc_text &txt, chunk_t *pc)
    {
       return;
    }
-
    txt += cur->str;
    cur  = chunk_get_next_ncnl(cur);
 
@@ -959,15 +1060,15 @@ void add_long_closebrace_comment(void)
          cl_pc = pc;
       }
 
-      if (pc->type != CT_BRACE_OPEN || (pc->flags & PCF_IN_PREPROC))
+      if (pc->type != CT_BRACE_OPEN || pc->flags.test(PCF_IN_PREPROC))
       {
          continue;
       }
-
       chunk_t *br_open = pc;
       size_t  nl_count = 0;
 
       chunk_t *tmp = pc;
+
       while ((tmp = chunk_get_next(tmp, scope_e::PREPROC)) != nullptr)
       {
          if (chunk_is_newline(tmp))
@@ -975,6 +1076,7 @@ void add_long_closebrace_comment(void)
             nl_count += tmp->nl_count;
             continue;
          }
+
          // handle only matching closing braces, skip other chunks
          if (tmp->level != br_open->level || tmp->type != CT_BRACE_CLOSE)
          {
@@ -992,20 +1094,21 @@ void add_long_closebrace_comment(void)
             br_close = tmp;
             tmp      = chunk_get_next(tmp);
          }
+
          // make sure a newline follows in order to not overwrite an already
          // existring comment
          if (tmp != nullptr && !chunk_is_newline(tmp))
          {
             break;
          }
-
          size_t   nl_min  = 0;
          chunk_t  *tag_pc = nullptr;
          unc_text xstr;
 
-         if (  br_open->parent_type == CT_FUNC_DEF
-            || br_open->parent_type == CT_OC_MSG_DECL)
+         if (  get_chunk_parent_type(br_open) == CT_FUNC_DEF
+            || get_chunk_parent_type(br_open) == CT_OC_MSG_DECL)
          {
+            log_rule_B("mod_add_long_function_closebrace_comment");
             nl_min = options::mod_add_long_function_closebrace_comment();
             tag_pc = fcn_pc;
 
@@ -1014,14 +1117,16 @@ void add_long_closebrace_comment(void)
                append_tag_name(xstr, tag_pc);
             }
          }
-         else if (br_open->parent_type == CT_SWITCH && sw_pc != nullptr)
+         else if (get_chunk_parent_type(br_open) == CT_SWITCH && sw_pc != nullptr)
          {
+            log_rule_B("mod_add_long_switch_closebrace_comment");
             nl_min = options::mod_add_long_switch_closebrace_comment();
             tag_pc = sw_pc;
             xstr   = sw_pc->str;
          }
-         else if (br_open->parent_type == CT_NAMESPACE && ns_pc != nullptr)
+         else if (get_chunk_parent_type(br_open) == CT_NAMESPACE && ns_pc != nullptr)
          {
+            log_rule_B("mod_add_long_namespace_closebrace_comment");
             nl_min = options::mod_add_long_namespace_closebrace_comment();
             tag_pc = ns_pc;
             xstr   = tag_pc->str;    // add 'namespace' to the string
@@ -1029,6 +1134,7 @@ void add_long_closebrace_comment(void)
             // next chunk, normally is going to be the namespace name
             // append it with a space to generate "namespace xyz"
             chunk_t *tmp_next = chunk_get_next_ncnl(tag_pc);
+
             if (  tmp_next != nullptr
                && tmp_next->type != CT_BRACE_OPEN) // anonymous namespace -> ignore
             {
@@ -1036,16 +1142,18 @@ void add_long_closebrace_comment(void)
                append_tag_name(xstr, tmp_next);
             }
          }
-         else if (  br_open->parent_type == CT_CLASS
+         else if (  get_chunk_parent_type(br_open) == CT_CLASS
                  && cl_pc != nullptr
                  && (  !language_is_set(LANG_CPP)                 // proceed if not C++
                     || (chunk_is_token(br_close, CT_SEMICOLON)))) // else a C++ class needs to end with a semicolon
          {
+            log_rule_B("mod_add_long_class_closebrace_comment");
             nl_min = options::mod_add_long_class_closebrace_comment();
             tag_pc = cl_pc;
             xstr   = tag_pc->str;
 
             chunk_t *tmp_next = chunk_get_next(cl_pc);
+
             if (tag_pc != nullptr)
             {
                xstr.append(" ");
@@ -1064,7 +1172,6 @@ void add_long_closebrace_comment(void)
             // Add a comment after the close brace
             insert_comment_after(br_close, style, xstr);
          }
-
          break;
       }
    }
@@ -1080,7 +1187,7 @@ static void move_case_break(void)
    {
       if (  chunk_is_token(pc, CT_BREAK)
          && chunk_is_token(prev, CT_BRACE_CLOSE)
-         && prev->parent_type == CT_CASE
+         && get_chunk_parent_type(prev) == CT_CASE
          && chunk_is_newline(chunk_get_prev(pc))
          && chunk_is_newline(chunk_get_prev(prev)))
       {
@@ -1100,14 +1207,15 @@ static chunk_t *mod_case_brace_remove(chunk_t *br_open)
    // Find the matching brace close
    chunk_t *next     = chunk_get_next_ncnl(br_open, scope_e::PREPROC);
    chunk_t *br_close = chunk_get_next_type(br_open, CT_BRACE_CLOSE, br_open->level, scope_e::PREPROC);
+
    if (br_close == nullptr)
    {
       LOG_FMT(LMCB, "%s(%d):  - no close\n", __func__, __LINE__);
       return(next);
    }
-
    // Make sure 'break', 'return', 'goto', 'case' or '}' is after the close brace
    chunk_t *pc = chunk_get_next_ncnl(br_close, scope_e::PREPROC);
+
    if (  pc == nullptr
       || (  pc->type != CT_BREAK
          && pc->type != CT_RETURN
@@ -1125,13 +1233,14 @@ static chunk_t *mod_case_brace_remove(chunk_t *br_open)
         tmp_pc != br_close;
         tmp_pc = chunk_get_next_ncnl(tmp_pc, scope_e::PREPROC))
    {
-      if (tmp_pc->level == (br_open->level + 1) && (tmp_pc->flags & PCF_VAR_DEF))
+      if (tmp_pc->level == (br_open->level + 1) && tmp_pc->flags.test(PCF_VAR_DEF))
       {
          LOG_FMT(LMCB, "%s(%d):  - vardef on line %zu: '%s'\n",
                  __func__, __LINE__, tmp_pc->orig_line, pc->text());
          return(next);
       }
    }
+
    LOG_FMT(LMCB, "%s(%d):  - removing braces on lines %zu and %zu\n",
            __func__, __LINE__, br_open->orig_line, br_close->orig_line);
 
@@ -1139,9 +1248,25 @@ static chunk_t *mod_case_brace_remove(chunk_t *br_open)
         tmp_pc != br_close;
         tmp_pc = chunk_get_next_ncnl(tmp_pc, scope_e::PREPROC))
    {
+      if (tmp_pc->brace_level == 0)
+      {
+         fprintf(stderr, "%s(%d): tmp_pc->brace_level is ZERO, cannot be decremented, at line %zu, column %zu\n",
+                 __func__, __LINE__, tmp_pc->orig_line, tmp_pc->orig_col);
+         log_flush(true);
+         exit(EX_SOFTWARE);
+      }
       tmp_pc->brace_level--;
+
+      if (tmp_pc->level == 0)
+      {
+         fprintf(stderr, "%s(%d): tmp_pc->level is ZERO, cannot be decremented, at line %zu, column %zu\n",
+                 __func__, __LINE__, tmp_pc->orig_line, tmp_pc->orig_col);
+         log_flush(true);
+         exit(EX_SOFTWARE);
+      }
       tmp_pc->level--;
    }
+
    next = chunk_get_prev(br_open, scope_e::PREPROC);
 
    chunk_del(br_open);
@@ -1182,15 +1307,15 @@ static chunk_t *mod_case_brace_add(chunk_t *cl_colon)
       LOG_FMT(LMCB, "%s(%d):  - last is nullptr\n", __func__, __LINE__);
       return(next);
    }
-
    LOG_FMT(LMCB, "%s(%d):  - adding before '%s' on line %zu\n",
            __func__, __LINE__, last->text(), last->orig_line);
 
    chunk_t chunk;
-   chunk.type        = CT_BRACE_OPEN;
+
+   set_chunk_type(&chunk, CT_BRACE_OPEN);
+   set_chunk_parent(&chunk, CT_CASE);
    chunk.orig_line   = cl_colon->orig_line;
    chunk.orig_col    = cl_colon->orig_col;
-   chunk.parent_type = CT_CASE;
    chunk.level       = cl_colon->level;
    chunk.brace_level = cl_colon->brace_level;
    chunk.flags       = pc->flags & PCF_COPY_FLAGS;
@@ -1198,12 +1323,13 @@ static chunk_t *mod_case_brace_add(chunk_t *cl_colon)
 
    chunk_t *br_open = chunk_add_after(&chunk, cl_colon);
 
-   chunk.type      = CT_BRACE_CLOSE;
+   set_chunk_type(&chunk, CT_BRACE_CLOSE);
    chunk.orig_line = last->orig_line;
    chunk.orig_col  = last->orig_col;
    chunk.str       = "}";
 
    chunk_t *br_close = chunk_add_before(&chunk, last);
+
    newline_add_before(last);
 
    for (pc = chunk_get_next(br_open, scope_e::PREPROC);
@@ -1223,17 +1349,20 @@ static void mod_case_brace(void)
    LOG_FUNC_ENTRY();
 
    chunk_t *pc = chunk_get_head();
+
    while (pc != nullptr)
    {
       chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
+
       if (next == nullptr)
       {
          return;
       }
+      log_rule_B("mod_case_brace");
 
       if (  options::mod_case_brace() == IARF_REMOVE
          && chunk_is_token(pc, CT_BRACE_OPEN)
-         && pc->parent_type == CT_CASE)
+         && get_chunk_parent_type(pc) == CT_CASE)
       {
          pc = mod_case_brace_remove(pc);
       }
@@ -1243,6 +1372,7 @@ static void mod_case_brace(void)
               && next->type != CT_BRACE_CLOSE
               && next->type != CT_CASE)
       {
+         log_rule_B("mod_case_brace");
          pc = mod_case_brace_add(pc);
       }
       else
@@ -1260,21 +1390,25 @@ static void process_if_chain(chunk_t *br_start)
            __func__, __LINE__, br_start->orig_line, br_start->orig_col);
 
    vector<chunk_t *> braces;
+
    braces.reserve(16);
 
    bool    must_have_braces = false;
 
    chunk_t *pc = br_start;
+
    while (pc != nullptr)
    {
       LOG_FMT(LBRCH, "%s(%d): pc->text() is '%s', orig_line is %zu, orig_col is %zu.\n",
               __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
+
       if (chunk_is_token(pc, CT_BRACE_OPEN))
       {
          const bool tmp = can_remove_braces(pc);
          LOG_FMT(LBRCH, "%s(%d): braces.size() is %zu, line is %zu, - can%s remove %s\n",
                  __func__, __LINE__, braces.size(), pc->orig_line, tmp ? "" : "not",
                  get_token_name(pc->type));
+
          if (!tmp)
          {
             must_have_braces = true;
@@ -1283,6 +1417,7 @@ static void process_if_chain(chunk_t *br_start)
       else
       {
          const bool tmp = should_add_braces(pc);
+
          if (tmp)
          {
             must_have_braces = true;
@@ -1291,9 +1426,9 @@ static void process_if_chain(chunk_t *br_start)
                  __func__, __LINE__, braces.size(), pc->orig_line, tmp ? "should add" : "ignore",
                  get_token_name(pc->type));
       }
-
       braces.push_back(pc);
       chunk_t *br_close = chunk_skip_to_match(pc, scope_e::PREPROC);
+
       if (br_close == nullptr)
       {
          break;
@@ -1301,18 +1436,20 @@ static void process_if_chain(chunk_t *br_start)
       braces.push_back(br_close);
 
       pc = chunk_get_next_ncnl(br_close, scope_e::PREPROC);
+
       if (pc == nullptr || pc->type != CT_ELSE)
       {
          break;
       }
+      log_rule_B("mod_full_brace_if_chain_only");
 
       if (options::mod_full_brace_if_chain_only())
       {
          // There is an 'else' - we want full braces.
          must_have_braces = true;
       }
-
       pc = chunk_get_next_ncnl(pc, scope_e::PREPROC);
+
       if (chunk_is_token(pc, CT_ELSEIF))
       {
          while (  pc != nullptr
@@ -1322,10 +1459,12 @@ static void process_if_chain(chunk_t *br_start)
             pc = chunk_get_next_ncnl(pc, scope_e::PREPROC);
          }
       }
+
       if (pc == nullptr)
       {
          break;
       }
+
       if (pc->type != CT_BRACE_OPEN && pc->type != CT_VBRACE_OPEN)
       {
          break;
@@ -1338,11 +1477,13 @@ static void process_if_chain(chunk_t *br_start)
               __func__, __LINE__, braces.size());
 
       const auto ite = braces.rend();
+
       for (auto itc = braces.rbegin(); itc != ite; ++itc)
       {
          const auto brace = *itc;
 
          chunk_flags_set(brace, PCF_KEEP_BRACE);
+
          if (chunk_is_token(brace, CT_VBRACE_OPEN) || chunk_is_token(brace, CT_VBRACE_CLOSE))
          {
             LOG_FMT(LBRCH, "%s(%d):  %zu",
@@ -1355,10 +1496,12 @@ static void process_if_chain(chunk_t *br_start)
                     __func__, __LINE__, brace->orig_line);
          }
       }
+
       LOG_FMT(LBRCH, "\n");
    }
    else if (options::mod_full_brace_if_chain())
    {
+      log_rule_B("mod_full_brace_if_chain");
       LOG_FMT(LBRCH, "%s(%d): remove braces on lines[%zu]:\n",
               __func__, __LINE__, braces.size());
 
@@ -1368,17 +1511,20 @@ static void process_if_chain(chunk_t *br_start)
        * is used.
        * We only want to remove braces if the first one is active.
        */
+      log_rule_B("mod_full_brace_nl_block_rem_mlcond");
       const auto multiline_block = options::mod_full_brace_nl_block_rem_mlcond();
 
       LOG_FMT(LBRCH, "%s(%d): remove braces on lines:\n", __func__, __LINE__);
 
       // Issue #2229
       const auto ite = braces.end();
+
       for (auto itc = braces.begin(); itc != ite; ++itc)
       {
          const auto brace = *itc;
 
          if (  (chunk_is_token(brace, CT_BRACE_OPEN) || chunk_is_token(brace, CT_BRACE_CLOSE))
+            && (get_chunk_parent_type(brace) != CT_BRACED_INIT_LIST)
             && (multiline_block ? !paren_multiline_before_brace(brace) : true))
          {
             LOG_FMT(LBRCH, "%s(%d): brace->orig_line is %zu, brace->orig_col is %zu\n",
@@ -1402,7 +1548,7 @@ static void mod_full_brace_if_chain(void)
    for (chunk_t *pc = chunk_get_head(); pc != nullptr; pc = chunk_get_next(pc))
    {
       if (  (chunk_is_token(pc, CT_BRACE_OPEN) || chunk_is_token(pc, CT_VBRACE_OPEN))
-         && pc->parent_type == CT_IF)
+         && get_chunk_parent_type(pc) == CT_IF)
       {
          process_if_chain(pc);
       }
