@@ -5,10 +5,12 @@
  * @author  Ben Gardner
  * @license GPL v2+
  */
+
 #ifndef CHUNK_LIST_H_INCLUDED
 #define CHUNK_LIST_H_INCLUDED
 
 #include "uncrustify_types.h"
+// necessary to not sort it
 #include "char_table.h"
 #include "language_tools.h"
 
@@ -41,9 +43,6 @@ enum class scope_e : unsigned int
 };
 
 
-void set_chunk_real(chunk_t *pc, c_token_t token, log_sev_t what);
-
-
 /**
  * duplicate a chunk in a chunk list
  *
@@ -89,7 +88,12 @@ chunk_t *chunk_add_before(const chunk_t *pc_in, chunk_t *ref);
  *
  * @param pc  chunk to delete
  */
-void chunk_del(chunk_t *pc);
+#define chunk_del(pc)    do { \
+      chunk_del_2((pc));      \
+      (pc) = nullptr;         \
+} while (false)
+
+void chunk_del_2(chunk_t *pc);
 
 
 /**
@@ -189,7 +193,7 @@ chunk_t *chunk_get_next_nc(chunk_t *cur, scope_e scope = scope_e::ALL);
 
 
 /**
- * Gets the next non-NEWLINE and non-comment chunk
+ * Gets the next non-NEWLINE
  *
  * @param cur    chunk to use as start point
  * @param scope  code region to search in
@@ -416,6 +420,16 @@ chunk_t *chunk_get_next_ssq(chunk_t *cur);
  */
 chunk_t *chunk_get_prev_ssq(chunk_t *cur);
 
+/**
+ * Gets the corresponding start chunk if the given chunk is within a
+ * preprocessor directive, or nullptr otherwise.
+ *
+ * @param  cur    chunk to use as start point
+ *
+ * @return nullptr or start chunk of the preprocessor directive
+ */
+chunk_t *chunk_get_pp_start(chunk_t *cur);
+
 /**
  * @brief reverse search a chunk of a given category in a chunk list
  *
@@ -533,7 +547,7 @@ static inline chunk_t *chunk_skip_to_match_rev(chunk_t *cur, scope_e scope = sco
 
 
 //! skip to the final word/type in a :: chain
-chunk_t *chunk_skip_dc_member(chunk_t *start, scope_e scope     = scope_e::ALL);
+chunk_t *chunk_skip_dc_member(chunk_t *start, scope_e scope = scope_e::ALL);
 chunk_t *chunk_skip_dc_member_rev(chunk_t *start, scope_e scope = scope_e::ALL);
 
 
@@ -609,7 +623,7 @@ static inline bool chunk_is_balanced_square(chunk_t *pc)
 
 static inline bool chunk_is_preproc(chunk_t *pc)
 {
-   return(pc != nullptr && (pc->flags & PCF_IN_PREPROC));
+   return(pc != nullptr && pc->flags.test(PCF_IN_PREPROC));
 }
 
 
@@ -644,6 +658,7 @@ static inline bool chunk_is_Doxygen_comment(chunk_t *pc)
    // check the third character
    const char   *sComment = pc->text();
    const size_t len       = strlen(sComment);
+
    if (len < 3)
    {
       return(false);
@@ -720,15 +735,13 @@ static inline bool chunk_is_addr(chunk_t *pc)
    {
       chunk_t *prev = chunk_get_prev(pc);
 
-      if (  (pc->flags & PCF_IN_TEMPLATE)
+      if (  pc->flags.test(PCF_IN_TEMPLATE)
          && (chunk_is_token(prev, CT_COMMA) || chunk_is_token(prev, CT_ANGLE_OPEN)))
       {
          return(false);
       }
-
       return(true);
    }
-
    return(false);
 }
 
@@ -833,15 +846,18 @@ static inline bool chunk_is_forin(chunk_t *pc)
       && chunk_is_token(pc, CT_SPAREN_OPEN))
    {
       chunk_t *prev = chunk_get_prev_ncnl(pc);
+
       if (chunk_is_token(prev, CT_FOR))
       {
          chunk_t *next = pc;
+
          while (  next != nullptr
                && next->type != CT_SPAREN_CLOSE
                && next->type != CT_IN)
          {
             next = chunk_get_next_ncnl(next);
          }
+
          if (chunk_is_token(next, CT_IN))
          {
             return(true);
@@ -852,40 +868,44 @@ static inline bool chunk_is_forin(chunk_t *pc)
 }
 
 
-void set_chunk_type_real(chunk_t *pc, c_token_t tt);
+void set_chunk_type_real(chunk_t *pc, c_token_t tt, const char *func, int line);
 
 
-void set_chunk_parent_real(chunk_t *pc, c_token_t tt);
+void set_chunk_parent_real(chunk_t *pc, c_token_t tt, const char *func, int line);
 
 
-#define set_chunk_type(pc, tt)      do { \
-      LOG_FUNC_CALL();                   \
-      set_chunk_type_real((pc), (tt));   \
+#define set_chunk_type(pc, tt)      do {                   \
+      set_chunk_type_real((pc), (tt), __func__, __LINE__); \
 } while (false)
 
-#define set_chunk_parent(pc, tt)    do { \
-      LOG_FUNC_CALL();                   \
-      set_chunk_parent_real((pc), (tt)); \
+#define set_chunk_parent(pc, tt)    do {                     \
+      set_chunk_parent_real((pc), (tt), __func__, __LINE__); \
 } while (false)
 
 
-void chunk_flags_set_real(chunk_t *pc, UINT64 clr_bits, UINT64 set_bits);
+c_token_t get_chunk_parent_type(chunk_t *pc);
+
+
+void chunk_flags_set_real(chunk_t *pc, pcf_flags_t clr_bits, pcf_flags_t set_bits);
 
 
 #define chunk_flags_upd(pc, cc, ss)    do {   \
-      LOG_FUNC_CALL();                        \
       chunk_flags_set_real((pc), (cc), (ss)); \
 } while (false)
 
 #define chunk_flags_set(pc, ss)        do { \
-      LOG_FUNC_CALL();                      \
-      chunk_flags_set_real((pc), 0, (ss));  \
+      chunk_flags_set_real((pc), {}, (ss)); \
 } while (false)
 
 #define chunk_flags_clr(pc, cc)        do { \
-      LOG_FUNC_CALL();                      \
-      chunk_flags_set_real((pc), (cc), 0);  \
+      chunk_flags_set_real((pc), (cc), {}); \
 } while (false)
 
 
+void chunk_set_parent(chunk_t *pc, chunk_t *parent);
+
+
+c_token_t get_type_of_the_parent(chunk_t *pc);
+
+
 #endif /* CHUNK_LIST_H_INCLUDED */
