@@ -5,14 +5,14 @@
  * @author  Ben Gardner
  * @license GPL v2+
  */
+
 #include "detect.h"
 
-#include "uncrustify_types.h"
 #include "chunk_list.h"
 #include "ChunkStack.h"
 #include "prototypes.h"
-
 #include "unc_ctype.h"
+#include "uncrustify_types.h"
 
 #include <cstdio>
 #include <cstdlib>
@@ -35,7 +35,8 @@ protected:
 
 public:
    sp_votes(Option<iarf_e> &opt)
-      : m_option(opt) {}
+      : m_option(opt)
+   {}
 
    //! Figure out the result of the vote and maybe update *m_av
    ~sp_votes();
@@ -53,8 +54,8 @@ void sp_votes::vote(chunk_t *first, chunk_t *second)
    {
       return;
    }
-
    int col_dif = second->column - (first->column + first->len());
+
    if (col_dif == 0)
    {
       m_remove++;
@@ -160,6 +161,7 @@ static void detect_space_options(void)
    while (pc != nullptr)
    {
       next = chunk_get_next(pc);
+
       if (next == nullptr)
       {
          break;
@@ -170,9 +172,10 @@ static void detect_space_options(void)
          vote_sp_arith.vote(pc, next);
          vote_sp_arith.vote(prev, pc);
       }
+
       if (chunk_is_token(pc, CT_ASSIGN))
       {
-         if ((pc->flags & PCF_IN_ENUM) == 0)
+         if (!pc->flags.test(PCF_IN_ENUM))
          {
             vote_sp_before_assign.vote(prev, pc);
             vote_sp_after_assign.vote(pc, next);
@@ -183,46 +186,56 @@ static void detect_space_options(void)
             vote_sp_enum_after_assign.vote(pc, next);
          }
       }
+
       if (chunk_is_token(pc, CT_SQUARE_OPEN))
       {
          vote_sp_before_square.vote(prev, pc);
          vote_sp_inside_square.vote(pc, next);
       }
+
       if (chunk_is_token(pc, CT_SQUARE_CLOSE))
       {
          vote_sp_inside_square.vote(prev, pc);
       }
+
       if (chunk_is_token(pc, CT_TSQUARE))
       {
          vote_sp_before_squares.vote(prev, pc);
       }
+
       if (chunk_is_token(pc, CT_BOOL))
       {
          vote_sp_bool.vote(prev, pc);
          vote_sp_bool.vote(pc, next);
       }
+
       if (chunk_is_token(pc, CT_COMPARE))
       {
          vote_sp_compare.vote(prev, pc);
          vote_sp_compare.vote(pc, next);
       }
+
       if (chunk_is_token(pc, CT_PAREN_CLOSE))
       {
          vote_sp_inside_paren.vote(prev, pc);
       }
+
       if (chunk_is_token(pc, CT_PAREN_OPEN))
       {
          vote_sp_inside_paren.vote(pc, next);
       }
+
       if (  (chunk_is_paren_open(pc) && chunk_is_paren_open(next))
          || (chunk_is_paren_close(pc) && chunk_is_paren_close(next)))
       {
          vote_sp_paren_paren.vote(pc, next);
       }
+
       if (chunk_is_paren_close(pc) && chunk_is_token(next, CT_BRACE_OPEN))
       {
          vote_sp_paren_brace.vote(pc, next);
       }
+
       if (chunk_is_token(pc, CT_PTR_TYPE))
       {
          if (chunk_is_token(prev, CT_PTR_TYPE))
@@ -237,11 +250,13 @@ static void detect_space_options(void)
          {
             vote_sp_before_ptr_star.vote(prev, pc);
          }
+
          if (CharTable::IsKw1(next->str[0]))
          {
             vote_sp_after_ptr_star.vote(pc, next);
          }
       }
+
       if (chunk_is_token(pc, CT_BYREF))
       {
          if (next->type != CT_WORD)
@@ -254,14 +269,17 @@ static void detect_space_options(void)
          }
          vote_sp_after_byref.vote(pc, next);
       }
+
       if (  pc->type != CT_PTR_TYPE
          && (chunk_is_token(prev, CT_QUALIFIER) || chunk_is_token(prev, CT_TYPE)))
       {
          vote_sp_after_type.vote(prev, pc);
       }
+
       if (chunk_is_token(pc, CT_ANGLE_OPEN))
       {
          vote_sp_inside_angle.vote(pc, next);
+
          if (chunk_is_token(prev, CT_TEMPLATE))
          {
             vote_sp_template_angle.vote(prev, pc);
@@ -271,9 +289,11 @@ static void detect_space_options(void)
             vote_sp_before_angle.vote(prev, pc);
          }
       }
+
       if (chunk_is_token(pc, CT_ANGLE_CLOSE))
       {
          vote_sp_inside_angle.vote(prev, pc);
+
          if (chunk_is_paren_open(next))
          {
             vote_sp_angle_paren.vote(prev, pc);
@@ -293,9 +313,11 @@ static void detect_space_options(void)
          vote_sp_before_sparen.vote(prev, pc);
          vote_sp_inside_sparen.vote(pc, next);
       }
+
       if (chunk_is_token(pc, CT_SPAREN_CLOSE))
       {
          vote_sp_inside_sparen.vote(prev, pc);
+
          if (chunk_is_token(next, CT_BRACE_OPEN))
          {
             vote_sp_sparen_brace.vote(pc, next);
@@ -305,9 +327,10 @@ static void detect_space_options(void)
             vote_sp_after_sparen.vote(pc, next);
          }
       }
+
       if (chunk_is_token(pc, CT_SEMICOLON))
       {
-         if (pc->parent_type == CT_FOR)
+         if (get_chunk_parent_type(pc) == CT_FOR)
          {
             if (chunk_is_token(prev, CT_SPAREN_OPEN))
             {
@@ -341,16 +364,19 @@ static void detect_space_options(void)
             vote_sp_before_semi.vote(prev, pc);
          }
       }
+
       if (chunk_is_token(pc, CT_COMMA))
       {
          vote_sp_before_comma.vote(prev, pc);
          vote_sp_after_comma.vote(pc, next);
       }
+
       if (chunk_is_token(pc, CT_CLASS_COLON))
       {
          vote_sp_before_class_colon.vote(prev, pc);
          vote_sp_after_class_colon.vote(pc, next);
       }
+
       if (chunk_is_token(pc, CT_BRACE_OPEN))
       {
          if (chunk_is_token(prev, CT_ELSE))
@@ -373,6 +399,7 @@ static void detect_space_options(void)
          {
             vote_sp_catch_brace.vote(prev, pc);
          }
+
          if (chunk_is_token(next, CT_BRACE_CLOSE))
          {
             vote_sp_inside_braces_empty.vote(pc, next);
@@ -382,9 +409,11 @@ static void detect_space_options(void)
             vote_sp_inside_braces.vote(pc, next);
          }
       }
+
       if (chunk_is_token(pc, CT_BRACE_CLOSE))
       {
          vote_sp_inside_braces.vote(prev, pc);
+
          if (chunk_is_token(next, CT_ELSE))
          {
             vote_sp_brace_else.vote(pc, next);
@@ -398,7 +427,6 @@ static void detect_space_options(void)
             vote_sp_brace_finally.vote(pc, next);
          }
       }
-
       prev = pc;
       pc   = next;
    }
