@@ -7,18 +7,21 @@
  *          October 2015, 2016
  * @license GPL v2+
  */
+
 #include "keywords.h"
-#include "uncrustify_types.h"
-#include "prototypes.h"
-#include "char_table.h"
+
 #include "args.h"
-#include <cstring>
-#include <cerrno>
-#include <cstdlib>
-#include <map>
+#include "char_table.h"
+#include "language_tools.h"
+#include "prototypes.h"
 #include "unc_ctype.h"
 #include "uncrustify.h"
-#include "language_tools.h"
+#include "uncrustify_types.h"
+
+#include <cerrno>
+#include <cstdlib>
+#include <cstring>
+#include <map>
 
 using namespace std;
 
@@ -58,6 +61,7 @@ static const chunk_tag_t *kw_static_match(const chunk_tag_t *tag, int lang_flags
 static chunk_tag_t keywords[] =
 {
    // TODO: it might be useful if users could add there custom keywords to this list
+   { "@autoreleasepool",                CT_AUTORELEASEPOOL,  LANG_OC                                                                     },
    { "@available",                      CT_OC_AVAILABLE,     LANG_OC                                                                     },
    { "@catch",                          CT_CATCH,            LANG_OC                                                                     },
    { "@dynamic",                        CT_OC_DYNAMIC,       LANG_OC                                                                     },
@@ -293,6 +297,7 @@ static chunk_tag_t keywords[] =
    { "signal",                          CT_ACCESS,           LANG_VALA                                                                   },
    { "signals",                         CT_ACCESS,           LANG_CPP                                                                    },
    { "signed",                          CT_TYPE,             LANG_C                                                                      },
+   { "size_t",                          CT_TYPE,             LANG_ALLC                                                                   },
    { "sizeof",                          CT_SIZEOF,           LANG_C | LANG_CS | LANG_VALA | LANG_PAWN                                    }, // PAWN
    { "sleep",                           CT_SIZEOF,           LANG_PAWN                                                                   }, // PAWN
    { "stackalloc",                      CT_NEW,              LANG_CS                                                                     },
@@ -369,13 +374,14 @@ void init_keywords()
       {
          int               lang_flags = LANG_OC;
          const chunk_tag_t *probe     = kw_static_match(tag, lang_flags);
+
          if (probe == NULL)
          {
             tag->lang_flags |= lang_flags;
          }
-
          lang_flags = LANG_CPP;
          probe      = kw_static_match(tag, lang_flags);
+
          if (probe == NULL)
          {
             tag->lang_flags |= lang_flags;
@@ -426,7 +432,6 @@ void add_keyword(const std::string &tag, c_token_t type)
       (*it).second = type;
       return;
    }
-
    // Insert the keyword
    dkwm.insert(dkwmap::value_type(tag, type));
    LOG_FMT(LDYNKW, "%s(%d): added '%s' as '%s'\n",
@@ -460,6 +465,7 @@ static const chunk_tag_t *kw_static_match(const chunk_tag_t *tag, int lang_flags
         iter++)
    {
       bool pp_iter = (iter->lang_flags & FLAG_PP) != 0; // forcing value to bool
+
       if (  (strcmp(iter->tag, tag->tag) == 0)
          && language_is_set(iter->lang_flags)
          && (lang_flags & iter->lang_flags)
@@ -468,6 +474,7 @@ static const chunk_tag_t *kw_static_match(const chunk_tag_t *tag, int lang_flags
          return(iter);
       }
    }
+
    return(nullptr);
 }
 
@@ -478,16 +485,16 @@ c_token_t find_keyword_type(const char *word, size_t len)
    {
       return(CT_NONE);
    }
-
    // check the dynamic word list first
    string           ss(word, len);
    dkwmap::iterator it = dkwm.find(ss);
+
    if (it != dkwm.end())
    {
       return((*it).second);
    }
-
    chunk_tag_t key;
+
    key.tag = ss.c_str();
 
    // check the static word list
@@ -517,7 +524,6 @@ int load_keyword_file(const char *filename)
       cpd.error_count++;
       return(EX_IOERR);
    }
-
 #define MAXLENGTHOFLINE    256
 #define NUMBEROFARGS       2
    // maximal length of a line in the file
@@ -532,11 +538,11 @@ int load_keyword_file(const char *filename)
 
       // remove comments after '#' sign
       char *ptr;
+
       if ((ptr = strchr(buf, '#')) != nullptr)
       {
          *ptr = 0; // set string end where comment begins
       }
-
       size_t argc = Args::SplitLine(buf, args, NUMBEROFARGS);
 
       if (argc > 0)
@@ -557,7 +563,6 @@ int load_keyword_file(const char *filename)
          continue; // the line is empty
       }
    }
-
    fclose(pf);
    return(EX_OK);
 } // load_keyword_file
@@ -568,6 +573,7 @@ void print_keywords(FILE *pfile)
    for (const auto &keyword_pair : dkwm)
    {
       c_token_t tt = keyword_pair.second;
+
       if (tt == CT_TYPE)
       {
          fprintf(pfile, "type %*.s%s\n",
