@@ -11,14 +11,15 @@
  */
 
 #include "compat.h"
-#include "logger.h"
-#include "uncrustify_types.h"
-#include "unc_ctype.h"
 #include "log_levels.h"
+#include "logger.h"
+#include "unc_ctype.h"
+#include "uncrustify_types.h"
+
 #include <cstdio>
 #include <deque>
-#include <vector>
 #include <stdarg.h>
+#include <vector>
 
 
 struct log_fcn_info
@@ -134,11 +135,11 @@ void log_flush(bool force_nl)
          g_log.bufX[g_log.buf_len]   = 0;
       }
       size_t retlength = fwrite(&g_log.bufX[0], g_log.buf_len, 1, g_log.log_file);
+
       if (retlength != 1)
       {
          // maybe we should log something to complain... =)
       }
-
       g_log.buf_len = 0;
    }
 }
@@ -161,7 +162,6 @@ static size_t log_start(log_sev_t sev)
    {
       g_log.buf_len = static_cast<size_t>(snprintf(&g_log.bufX[0], g_log.bufX.size(), "<%d>", sev));
    }
-
    size_t cap = (g_log.bufX.size() - 2) - g_log.buf_len;
 
    return((cap > 0) ? cap : 0);
@@ -171,6 +171,7 @@ static size_t log_start(log_sev_t sev)
 static void log_end(void)
 {
    g_log.in_log = (g_log.bufX[g_log.buf_len - 1] != '\n');
+
    if (  !g_log.in_log
       || (g_log.buf_len > (g_log.bufX.size() / 2)))
    {
@@ -179,30 +180,6 @@ static void log_end(void)
 }
 
 
-void log_str(log_sev_t sev, const char *str, size_t len)
-{
-   if (  str == nullptr
-      || len == 0
-      || !log_sev_on(sev))
-   {
-      return;
-   }
-
-   size_t cap = log_start(sev);
-   if (cap > 0)
-   {
-      if (len > cap)
-      {
-         len = cap;
-      }
-      memcpy(&g_log.bufX[g_log.buf_len], str, len);
-      g_log.buf_len            += len;
-      g_log.bufX[g_log.buf_len] = 0;
-   }
-   log_end();
-}
-
-
 void log_fmt(log_sev_t sev, const char *fmt, ...)
 {
    if (fmt == nullptr || !log_sev_on(sev))
@@ -215,11 +192,11 @@ void log_fmt(log_sev_t sev, const char *fmt, ...)
    {
       return;
    }
-
 #define BUFFERLENGTH    200
    char         buf[BUFFERLENGTH];
    // it MUST be a 'unsigned int' variable to be runable under windows
    unsigned int length = strlen(fmt);
+
    if (length >= BUFFERLENGTH)
    {
       fprintf(stderr, "FATAL: The variable 'buf' is not big enought:\n");
@@ -267,113 +244,10 @@ void log_fmt(log_sev_t sev, const char *fmt, ...)
          }
       }
    }
-
    log_end();
 } // log_fmt
 
 
-void log_hex(log_sev_t sev, const void *vdata, size_t len)
-{
-   if (vdata == nullptr || !log_sev_on(sev))
-   {
-      return;
-   }
-
-#define MAX_BUF    80
-   char        buf[MAX_BUF];
-   const UINT8 *dat = static_cast<const UINT8 *>(vdata);
-   size_t      idx  = 0;
-   while (len-- > 0)
-   {
-      buf[idx++] = to_hex_char(*dat >> 4);
-      buf[idx++] = to_hex_char(*dat);
-      dat++;
-
-      // prevent an overflow
-      if (idx >= (MAX_BUF - 3))
-      {
-         buf[idx] = 0;
-         log_str(sev, buf, idx);
-         idx = 0;
-      }
-   }
-
-   if (idx > 0)
-   {
-      buf[idx] = 0;
-      log_str(sev, buf, idx);
-   }
-}
-
-
-void log_hex_blk(log_sev_t sev, const void *data, size_t len)
-{
-   if (data == nullptr || !log_sev_on(sev))
-   {
-      return;
-   }
-
-   static char buf[80] = "nnn | XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX | cccccccccccccccc\n";
-   const UINT8 *dat    = static_cast<const UINT8 *>(data);
-   int         str_idx = 0;
-   int         chr_idx = 0;
-
-   /*
-    * Dump the specified number of bytes in hex, 16 byte per line by
-    * creating a string and then calling log_str()
-    */
-
-   // Loop through the data of the current iov
-   int count = 0;
-   int total = 0;
-   for (size_t idx = 0; idx < len; idx++)
-   {
-      if (count == 0)
-      {
-         str_idx = 6;
-         chr_idx = 56;
-
-         buf[0] = to_hex_char(total >> 12);
-         buf[1] = to_hex_char(total >> 8);
-         buf[2] = to_hex_char(total >> 4);
-      }
-
-      int tmp = dat[idx];
-
-      buf[str_idx]     = to_hex_char(tmp >> 4);
-      buf[str_idx + 1] = to_hex_char(tmp);
-      str_idx         += 3;
-
-      buf[chr_idx++] = unc_isprint(tmp) ? tmp : '.';
-
-      total++;
-      count++;
-      if (count >= 16)
-      {
-         count = 0;
-         log_str(sev, buf, 73);
-      }
-   }
-
-   // Print partial line if any
-   if (count != 0)
-   {
-      // Clear out any junk
-      while (count < 16)
-      {
-         buf[str_idx]     = ' ';   // MSB hex
-         buf[str_idx + 1] = ' ';   // LSB hex
-         str_idx         += 3;
-
-         buf[chr_idx++] = ' ';
-
-         count++;
-      }
-      log_str(sev, buf, 73);
-   }
-} // log_hex_blk
-
-
 log_func::log_func(const char *name, int line)
 {
    g_fq.push_back(log_fcn_info(name, line));
@@ -386,16 +260,6 @@ log_func::~log_func()
 }
 
 
-void log_func_call(int line)
-{
-   // REVISIT: pass the __func__ and verify it matches the top entry?
-   if (!g_fq.empty())
-   {
-      g_fq.back().line = line;
-   }
-}
-
-
 void log_func_stack(log_sev_t sev, const char *prefix, const char *suffix, size_t skip_cnt)
 {
    UNUSED(skip_cnt);
@@ -408,19 +272,23 @@ void log_func_stack(log_sev_t sev, const char *prefix, const char *suffix, size_
    const char *sep      = "";
    size_t     g_fq_size = g_fq.size();
    size_t     begin_with;
+
    if (g_fq_size > (skip_cnt + 1))
    {
       begin_with = g_fq_size - (skip_cnt + 1);
+
       for (size_t idx = begin_with; idx != 0; idx--)
       {
          LOG_FMT(sev, "%s %s:%d", sep, g_fq[idx].name, g_fq[idx].line);
          sep = ",";
       }
+
       LOG_FMT(sev, "%s %s:%d", sep, g_fq[0].name, g_fq[0].line);
    }
 #else
    LOG_FMT(sev, "-DEBUG NOT SET-");
 #endif
+
    if (suffix)
    {
       LOG_FMT(sev, "%s", suffix);
