@@ -10,18 +10,23 @@
  *          October 2015, 2016
  * @license GPL v2+
  */
+
 #include "tokenize_cleanup.h"
-#include "uncrustify_types.h"
-#include "prototypes.h"
-#include "chunk_list.h"
+
 #include "char_table.h"
-#include "unc_ctype.h"
-#include "uncrustify.h"
-#include "space.h"
+#include "chunk_list.h"
 #include "combine.h"
+#include "combine_skip.h"
+#include "error_types.h"
 #include "keywords.h"
 #include "language_tools.h"
+#include "log_rules.h"
+#include "prototypes.h"
 #include "punctuators.h"
+#include "space.h"
+#include "unc_ctype.h"
+#include "uncrustify.h"
+#include "uncrustify_types.h"
 
 #include <vector>
 
@@ -54,7 +59,7 @@ static void check_template_args(chunk_t *start, chunk_t *end);
  *
  * @param start  chunk to start check at
  */
-static void check_template(chunk_t *start);
+static void check_template(chunk_t *start, bool in_type_cast);
 
 
 /**
@@ -95,9 +100,9 @@ static chunk_t *handle_double_angle_close(chunk_t *pc)
    {
       if (  chunk_is_token(pc, CT_ANGLE_CLOSE)
          && chunk_is_token(next, CT_ANGLE_CLOSE)
-         && pc->parent_type == CT_NONE
+         && get_chunk_parent_type(pc) == CT_NONE
          && (pc->orig_col_end + 1) == next->orig_col
-         && next->parent_type == CT_NONE)
+         && get_chunk_parent_type(next) == CT_NONE)
       {
          pc->str.append('>');
          set_chunk_type(pc, CT_ARITH);
@@ -125,13 +130,13 @@ void split_off_angle_close(chunk_t *pc)
    {
       return;
    }
-
    chunk_t nc = *pc;
+
    pc->str.resize(1);
    pc->orig_col_end = pc->orig_col + 1;
    set_chunk_type(pc, CT_ANGLE_CLOSE);
 
-   nc.type = ct->type;
+   set_chunk_type(&nc, ct->type);
    nc.str.pop_front();
    nc.orig_col++;
    nc.column++;
@@ -139,6 +144,134 @@ void split_off_angle_close(chunk_t *pc)
 }
 
 
+void tokenize_trailing_return_types(void)
+{
+   // Issue #2330
+   // auto max(int a, int b) -> int;
+   // Issue #2460
+   // auto f01() -> bool;
+   // auto f02() noexcept -> bool;
+   // auto f03() noexcept(true) -> bool;
+   // auto f04() noexcept(false) -> bool;
+   // auto f05() noexcept -> bool = delete;
+   // auto f06() noexcept(true) -> bool = delete;
+   // auto f07() noexcept(false) -> bool = delete;
+   // auto f11() const -> bool;
+   // auto f12() const noexcept -> bool;
+   // auto f13() const noexcept(true) -> bool;
+   // auto f14() const noexcept(false) -> bool;
+   // auto f15() const noexcept -> bool = delete;
+   // auto f16() const noexcept(true) -> bool = delete;
+   // auto f17() const noexcept(false) -> bool = delete;
+   // auto f21() throw() -> bool;
+   // auto f22() throw() -> bool = delete;
+   // auto f23() const throw() -> bool;
+   // auto f24() const throw() -> bool = delete;
+   chunk_t *pc;
+
+   for (pc = chunk_get_head(); pc != nullptr; pc = chunk_get_next_ncnl(pc))
+   {
+      LOG_FMT(LNOTE, "%s(%d): orig_line is %zu, orig_col is %zu, text() is '%s'\n",
+              __func__, __LINE__, pc->orig_line, pc->orig_col, pc->text());
+
+      if (  chunk_is_token(pc, CT_MEMBER)
+         && (strcmp(pc->text(), "->") == 0))
+      {
+         chunk_t *tmp = chunk_get_prev_ncnl(pc);
+         chunk_t *tmp_2;
+         chunk_t *open_paren;
+
+         if (chunk_is_token(tmp, CT_QUALIFIER))
+         {
+            // auto max(int a, int b) const -> int;
+            // auto f11() const -> bool;
+            tmp = chunk_get_prev_ncnl(tmp);
+         }
+         else if (chunk_is_token(tmp, CT_NOEXCEPT))
+         {
+            // noexcept is present
+            tmp_2 = chunk_get_prev_ncnl(tmp);
+
+            if (chunk_is_token(tmp_2, CT_QUALIFIER))
+            {
+               // auto f12() const noexcept -> bool;
+               // auto f15() const noexcept -> bool = delete;
+               tmp = chunk_get_prev_ncnl(tmp_2);
+            }
+            else
+            {
+               // auto f02() noexcept -> bool;
+               // auto f05() noexcept -> bool = delete;
+               tmp = tmp_2;
+            }
+         }
+         else if (chunk_is_token(tmp, CT_PAREN_CLOSE))
+         {
+            open_paren = chunk_get_prev_type(tmp, CT_PAREN_OPEN, tmp->level);
+            tmp        = chunk_get_prev_ncnl(open_paren);
+
+            if (chunk_is_token(tmp, CT_NOEXCEPT))
+            {
+               // noexcept is present
+               tmp_2 = chunk_get_prev_ncnl(tmp);
+
+               if (chunk_is_token(tmp_2, CT_QUALIFIER))
+               {
+                  // auto f13() const noexcept(true) -> bool;
+                  // auto f14() const noexcept(false) -> bool;
+                  // auto f16() const noexcept(true) -> bool = delete;
+                  // auto f17() const noexcept(false) -> bool = delete;
+                  tmp = chunk_get_prev_ncnl(tmp_2);
+               }
+               else
+               {
+                  // auto f03() noexcept(true) -> bool;
+                  // auto f04() noexcept(false) -> bool;
+                  // auto f06() noexcept(true) -> bool = delete;
+                  // auto f07() noexcept(false) -> bool = delete;
+                  tmp = tmp_2;
+               }
+            }
+            else if (chunk_is_token(tmp, CT_THROW))
+            {
+               // throw is present
+               tmp_2 = chunk_get_prev_ncnl(tmp);
+
+               if (chunk_is_token(tmp_2, CT_QUALIFIER))
+               {
+                  // auto f23() const throw() -> bool;
+                  // auto f24() const throw() -> bool = delete;
+                  tmp = chunk_get_prev_ncnl(tmp_2);
+               }
+               else
+               {
+                  // auto f21() throw() -> bool;
+                  // auto f22() throw() -> bool = delete;
+                  tmp = tmp_2;
+               }
+            }
+            else
+            {
+               LOG_FMT(LNOTE, "%s(%d): NOT COVERED\n", __func__, __LINE__);
+            }
+         }
+         else
+         {
+            LOG_FMT(LNOTE, "%s(%d): NOT COVERED\n", __func__, __LINE__);
+         }
+
+         if (  chunk_is_token(tmp, CT_FPAREN_CLOSE)
+            && (get_chunk_parent_type(tmp) == CT_FUNC_PROTO || get_chunk_parent_type(tmp) == CT_FUNC_DEF))
+         {
+            set_chunk_type(pc, CT_TRAILING_RET);
+            LOG_FMT(LNOTE, "%s(%d): set trailing return type for text() is '%s'\n",
+                    __func__, __LINE__, tmp->text());
+         }
+      }
+   }
+} // tokenize_trailing_return_types
+
+
 void tokenize_cleanup(void)
 {
    LOG_FUNC_ENTRY();
@@ -154,11 +287,13 @@ void tokenize_cleanup(void)
     * this change in the first pass.
     */
    chunk_t *pc;
+
    for (pc = chunk_get_head(); pc != nullptr; pc = chunk_get_next_ncnl(pc))
    {
       if (chunk_is_token(pc, CT_SQUARE_OPEN))
       {
          next = chunk_get_next_ncnl(pc);
+
          if (chunk_is_token(next, CT_SQUARE_CLOSE))
          {
             // Change '[' + ']' into '[]'
@@ -173,8 +308,9 @@ void tokenize_cleanup(void)
             chunk_del(next);
          }
       }
+
       if (  chunk_is_token(pc, CT_SEMICOLON)
-         && (pc->flags & PCF_IN_PREPROC)
+         && pc->flags.test(PCF_IN_PREPROC)
          && !chunk_get_next_ncnl(pc, scope_e::PREPROC))
       {
          LOG_FMT(LNOTE, "%s(%d): %s:%zu Detected a macro that ends with a semicolon. Possible failures if used.\n",
@@ -188,6 +324,7 @@ void tokenize_cleanup(void)
       if (chunk_is_token(pc, CT_COLON))
       {
          next = chunk_get_next_ncnl(pc);
+
          if (chunk_is_token(next, CT_ASSIGN))
          {
             // Change ':' + '=' into ':='
@@ -202,6 +339,7 @@ void tokenize_cleanup(void)
    // We can handle everything else in the second pass
    pc   = chunk_get_head();
    next = chunk_get_next_ncnl(pc);
+
    while (pc != nullptr && next != nullptr)
    {
       if (chunk_is_token(pc, CT_DOT) && language_is_set(LANG_ALLC))
@@ -259,24 +397,27 @@ void tokenize_cleanup(void)
       }
 
       if (  chunk_is_token(pc, CT_ENUM)
-         && chunk_is_token(next, CT_CLASS))
+         && (chunk_is_token(next, CT_STRUCT) || chunk_is_token(next, CT_CLASS)))
       {
          set_chunk_type(next, CT_ENUM_CLASS);
       }
+      chunk_t *next_non_attr = language_is_set(LANG_CPP) ? skip_attribute_next(next) : next;
 
       /*
-       * Change CT_WORD after CT_ENUM, CT_UNION, or CT_STRUCT to CT_TYPE
+       * Change CT_WORD after CT_ENUM, CT_UNION, CT_STRUCT, or CT_CLASS to CT_TYPE
        * Change CT_WORD before CT_WORD to CT_TYPE
        */
-      if (chunk_is_token(next, CT_WORD))
+      if (chunk_is_token(next_non_attr, CT_WORD))
       {
          if (  chunk_is_token(pc, CT_ENUM)
             || chunk_is_token(pc, CT_ENUM_CLASS)
             || chunk_is_token(pc, CT_UNION)
-            || chunk_is_token(pc, CT_STRUCT))
+            || chunk_is_token(pc, CT_STRUCT)
+            || chunk_is_token(pc, CT_CLASS))
          {
-            set_chunk_type(next, CT_TYPE);
+            set_chunk_type(next_non_attr, CT_TYPE);
          }
+
          if (chunk_is_token(pc, CT_WORD))
          {
             set_chunk_type(pc, CT_TYPE);
@@ -301,6 +442,7 @@ void tokenize_cleanup(void)
          {
             // Something else followed by a open brace
             chunk_t *tmp = chunk_get_next_ncnl(next);
+
             if (tmp == nullptr || tmp->type != CT_BRACE_OPEN)
             {
                set_chunk_type(pc, CT_QUALIFIER);
@@ -334,7 +476,8 @@ void tokenize_cleanup(void)
       }
 
       // Change angle open/close to CT_COMPARE, if not a template thingy
-      if (chunk_is_token(pc, CT_ANGLE_OPEN) && pc->parent_type != CT_TYPE_CAST)
+      if (  chunk_is_token(pc, CT_ANGLE_OPEN)
+         && pc->parent_type != CT_TYPE_CAST)
       {
          /*
           * pretty much all languages except C use <> for something other than
@@ -343,7 +486,7 @@ void tokenize_cleanup(void)
          if (language_is_set(LANG_OC | LANG_CPP | LANG_CS | LANG_JAVA | LANG_VALA))
          {
             // bug #663
-            check_template(pc);
+            check_template(pc, in_type_cast);
          }
          else
          {
@@ -351,7 +494,9 @@ void tokenize_cleanup(void)
             set_chunk_type(pc, CT_COMPARE);
          }
       }
-      if (chunk_is_token(pc, CT_ANGLE_CLOSE) && pc->parent_type != CT_TEMPLATE)
+
+      if (  chunk_is_token(pc, CT_ANGLE_CLOSE)
+         && pc->parent_type != CT_TEMPLATE)
       {
          if (in_type_cast)
          {
@@ -403,6 +548,7 @@ void tokenize_cleanup(void)
             pc->orig_line = prev->orig_line;
             chunk_t *to_be_deleted = prev;
             prev = chunk_get_prev_ncnl(prev);
+
             if (prev != nullptr)
             {
                chunk_del(to_be_deleted);
@@ -443,10 +589,29 @@ void tokenize_cleanup(void)
        * likewise, 'class' may be a member name in Java.
        */
       if (  chunk_is_token(pc, CT_CLASS)
-         && !CharTable::IsKw1(next->str[0])
-         && pc->next->type != CT_DC_MEMBER)
+         && !CharTable::IsKw1(next->str[0]))
       {
-         set_chunk_type(pc, CT_WORD);
+         if (  chunk_is_not_token(next, CT_DC_MEMBER)
+            && chunk_is_not_token(next, CT_ATTRIBUTE))                       // Issue #2570
+         {
+            set_chunk_type(pc, CT_WORD);
+         }
+         else if (  chunk_is_token(prev, CT_DC_MEMBER)
+                 || chunk_is_token(prev, CT_TYPE))
+         {
+            set_chunk_type(pc, CT_TYPE);
+         }
+         else if (chunk_is_token(next, CT_DC_MEMBER))
+         {
+            chunk_t *next2 = chunk_get_next_nblank(next);
+
+            if (  chunk_is_token(next2, CT_INV)      // CT_INV hasn't turned into CT_DESTRUCTOR just yet
+               || (  chunk_is_token(next2, CT_CLASS) // constructor isn't turned into CT_FUNC* just yet
+                  && !strcmp(pc->text(), next2->text())))
+            {
+               set_chunk_type(pc, CT_TYPE);
+            }
+         }
       }
 
       /*
@@ -469,10 +634,12 @@ void tokenize_cleanup(void)
       if (chunk_is_token(pc, CT_OPERATOR))
       {
          chunk_t *tmp2 = chunk_get_next(next);
+
          // Handle special case of () operator -- [] already handled
          if (chunk_is_token(next, CT_PAREN_OPEN))
          {
             chunk_t *tmp = chunk_get_next(next);
+
             if (chunk_is_token(tmp, CT_PAREN_CLOSE))
             {
                next->str = "()";
@@ -490,7 +657,7 @@ void tokenize_cleanup(void)
             set_chunk_type(next, CT_OPERATOR_VAL);
             chunk_del(tmp2);
          }
-         else if (next->flags & PCF_PUNCTUATOR)
+         else if (next->flags.test(PCF_PUNCTUATOR))
          {
             set_chunk_type(next, CT_OPERATOR_VAL);
          }
@@ -504,6 +671,7 @@ void tokenize_cleanup(void)
              */
             tmp2 = next;
             chunk_t *tmp;
+
             while ((tmp = chunk_get_next(tmp2)) != nullptr)
             {
                if (  tmp->type != CT_WORD
@@ -519,6 +687,7 @@ void tokenize_cleanup(void)
                // Change tmp into a type so that space_needed() works right
                make_type(tmp);
                size_t num_sp = space_needed(tmp2, tmp);
+
                while (num_sp-- > 0)
                {
                   next->str.append(" ");
@@ -531,7 +700,6 @@ void tokenize_cleanup(void)
             {
                chunk_del(tmp2);
             }
-
             set_chunk_type(next, CT_OPERATOR_VAL);
 
             next->orig_col_end = next->orig_col + next->len();
@@ -549,15 +717,18 @@ void tokenize_cleanup(void)
          if (chunk_is_str(next, "slots", 5) || chunk_is_str(next, "Q_SLOTS", 7))
          {
             chunk_t *tmp = chunk_get_next(next);
+
             if (chunk_is_token(tmp, CT_COLON))
             {
                next = tmp;
             }
          }
+
          if (chunk_is_token(next, CT_COLON))
          {
             set_chunk_type(next, CT_ACCESS_COLON);
             chunk_t *tmp;
+
             if ((tmp = chunk_get_next_ncnl(next)) != nullptr)
             {
                chunk_flags_set(tmp, PCF_STMT_START | PCF_EXPR_START);
@@ -579,11 +750,13 @@ void tokenize_cleanup(void)
                || ((chunk_is_token(pc, CT_STRING)) && (!pc->str.startswith("$\"")) && (!pc->str.startswith("$@\""))))))
       {
          chunk_t *tmp = chunk_get_prev(pc);
+
          if (chunk_is_newline(tmp))
          {
             if (*pc->str.c_str() == '$')
             {
                set_chunk_type(pc, CT_SQL_EXEC);
+
                if (pc->len() > 1)
                {
                   // SPLIT OFF '$'
@@ -593,7 +766,7 @@ void tokenize_cleanup(void)
                   pc->str.resize(1);
                   pc->orig_col_end = pc->orig_col + 1;
 
-                  nc.type = CT_SQL_WORD;
+                  set_chunk_type(&nc, CT_SQL_WORD);
                   nc.str.pop_front();
                   nc.orig_col++;
                   nc.column++;
@@ -603,6 +776,7 @@ void tokenize_cleanup(void)
                }
             }
             tmp = chunk_get_next(next);
+
             if (chunk_is_str_case(tmp, "BEGIN", 5))
             {
                set_chunk_type(pc, CT_SQL_BEGIN);
@@ -623,6 +797,7 @@ void tokenize_cleanup(void)
                {
                   break;
                }
+
                if (  (tmp->len() > 0)
                   && (  unc_isalpha(*tmp->str.c_str())
                      || (*tmp->str.c_str() == '$')))
@@ -645,10 +820,12 @@ void tokenize_cleanup(void)
          pc->orig_col_end = next->orig_col_end;
          chunk_del(next);
          next = chunk_get_next_ncnl(pc);
+
          // label the 'in'
          if (chunk_is_token(next, CT_PAREN_OPEN))
          {
             chunk_t *tmp = chunk_get_next_ncnl(next);
+
             while (tmp && tmp->type != CT_PAREN_CLOSE)
             {
                if (chunk_is_str(tmp, "in", 2))
@@ -674,6 +851,7 @@ void tokenize_cleanup(void)
          {
             set_chunk_type(pc, CT_WORD);
          }
+
          if (  chunk_is_token(pc, CT_DO)
             && (  chunk_is_token(prev, CT_MINUS)
                || chunk_is_token(next, CT_SQUARE_CLOSE)))
@@ -716,12 +894,13 @@ void tokenize_cleanup(void)
          set_chunk_parent(next, pc->type);
 
          chunk_t *tmp = chunk_get_next_ncnl(next);
+
          if (tmp != nullptr)
          {
             chunk_flags_set(tmp, PCF_STMT_START | PCF_EXPR_START);
          }
-
          tmp = chunk_get_next_type(pc, CT_OC_END, pc->level);
+
          if (tmp != nullptr)
          {
             set_chunk_parent(tmp, pc->type);
@@ -731,6 +910,7 @@ void tokenize_cleanup(void)
       if (chunk_is_token(pc, CT_OC_INTF))
       {
          chunk_t *tmp = chunk_get_next_ncnl(pc, scope_e::PREPROC);
+
          while (tmp != nullptr && tmp->type != CT_OC_END)
          {
             if (get_token_pattern_class(tmp->type) != pattern_class_e::NONE)
@@ -751,32 +931,33 @@ void tokenize_cleanup(void)
        *   @interface ClassName ()
        *   @implementation ClassName ()
        */
-      if (  (  pc->parent_type == CT_OC_IMPL
-            || pc->parent_type == CT_OC_INTF
+      if (  (  get_chunk_parent_type(pc) == CT_OC_IMPL
+            || get_chunk_parent_type(pc) == CT_OC_INTF
             || chunk_is_token(pc, CT_OC_CLASS))
          && chunk_is_token(next, CT_PAREN_OPEN))
       {
-         set_chunk_parent(next, pc->parent_type);
+         set_chunk_parent(next, get_chunk_parent_type(pc));
 
          chunk_t *tmp = chunk_get_next(next);
+
          if (tmp != nullptr && tmp->next != nullptr)
          {
             if (chunk_is_token(tmp, CT_PAREN_CLOSE))
             {
                //set_chunk_type(tmp, CT_OC_CLASS_EXT);
-               set_chunk_parent(tmp, pc->parent_type);
+               set_chunk_parent(tmp, get_chunk_parent_type(pc));
             }
             else
             {
                set_chunk_type(tmp, CT_OC_CATEGORY);
-               set_chunk_parent(tmp, pc->parent_type);
+               set_chunk_parent(tmp, get_chunk_parent_type(pc));
             }
          }
-
          tmp = chunk_get_next_type(pc, CT_PAREN_CLOSE, pc->level);
+
          if (tmp != nullptr)
          {
-            set_chunk_parent(tmp, pc->parent_type);
+            set_chunk_parent(tmp, get_chunk_parent_type(pc));
          }
       }
 
@@ -809,6 +990,7 @@ void tokenize_cleanup(void)
          set_chunk_parent(next, pc->type);
 
          chunk_t *tmp = chunk_get_next(next);
+
          if (tmp != nullptr)
          {
             set_chunk_type(tmp, CT_OC_SEL_NAME);
@@ -875,7 +1057,7 @@ void tokenize_cleanup(void)
          if (chunk_is_token(prev, CT_TYPE))
          {
             // Issue # 1002
-            if ((pc->flags & PCF_IN_TEMPLATE) == 0)
+            if (!pc->flags.test(PCF_IN_TEMPLATE))
             {
                set_chunk_type(pc, CT_BYREF);
             }
@@ -919,19 +1101,20 @@ void tokenize_cleanup(void)
 } // tokenize_cleanup
 
 
-static void check_template(chunk_t *start)
+static void check_template(chunk_t *start, bool in_type_cast)
 {
    LOG_FMT(LTEMPL, "%s(%d): orig_line %zu, orig_col %zu:\n",
            __func__, __LINE__, start->orig_line, start->orig_col);
 
    chunk_t *prev = chunk_get_prev_ncnl(start, scope_e::PREPROC);
+
    if (prev == nullptr)
    {
       return;
    }
-
    chunk_t *end;
    chunk_t *pc;
+
    if (chunk_is_token(prev, CT_TEMPLATE))
    {
       LOG_FMT(LTEMPL, "%s(%d): CT_TEMPLATE:\n", __func__, __LINE__);
@@ -939,6 +1122,7 @@ static void check_template(chunk_t *start)
       // We have: "template< ... >", which is a template declaration
       size_t level  = 1;
       size_t parens = 0;
+
       for (pc = chunk_get_next_ncnl(start, scope_e::PREPROC);
            pc != nullptr;
            pc = chunk_get_next_ncnl(pc, scope_e::PREPROC))
@@ -948,9 +1132,17 @@ static void check_template(chunk_t *start)
 
          if ((pc->str[0] == '>') && (pc->len() > 1))
          {
-            LOG_FMT(LTEMPL, "%s(%d): {split '%s' at orig_line %zu, orig_col %zu}\n",
-                    __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
-            split_off_angle_close(pc);
+            if (pc->str[1] == '=')                         // Issue #1462 and #2565
+            {
+               LOG_FMT(LTEMPL, "%s(%d): do not split '%s' at orig_line %zu, orig_col %zu\n",
+                       __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
+            }
+            else
+            {
+               LOG_FMT(LTEMPL, "%s(%d): {split '%s' at orig_line %zu, orig_col %zu}\n",
+                       __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
+               split_off_angle_close(pc);
+            }
          }
 
          if (pc->type == CT_PAREN_OPEN)
@@ -970,7 +1162,15 @@ static void check_template(chunk_t *start)
             }
             else if (chunk_is_str(pc, ">", 1))
             {
+               if (level == 0)
+               {
+                  fprintf(stderr, "%s(%d): level is ZERO, cannot be decremented, at line %zu, column %zu\n",
+                          __func__, __LINE__, pc->orig_line, pc->orig_col);
+                  log_flush(true);
+                  exit(EX_SOFTWARE);
+               }
                level--;
+
                if (level == 0)
                {
                   break;
@@ -978,6 +1178,7 @@ static void check_template(chunk_t *start)
             }
          }
       }
+
       end = pc;
    }
    else
@@ -996,14 +1197,13 @@ static void check_template(chunk_t *start)
          && prev->type != CT_COMMA
          && prev->type != CT_QUALIFIER
          && prev->type != CT_OPERATOR_VAL
-         && prev->parent_type != CT_OPERATOR)
+         && get_chunk_parent_type(prev) != CT_OPERATOR)
       {
          LOG_FMT(LTEMPL, "%s(%d): - after type %s + ( - Not a template\n",
                  __func__, __LINE__, get_token_name(prev->type));
          set_chunk_type(start, CT_COMPARE);
          return;
       }
-
       LOG_FMT(LTEMPL, "%s(%d): - prev->type is %s -\n",
               __func__, __LINE__, get_token_name(prev->type));
 
@@ -1011,6 +1211,7 @@ static void check_template(chunk_t *start)
       bool in_if         = false;
       bool hit_semicolon = false;
       pc = start;
+
       while ((pc = chunk_get_prev_ncnl(pc, scope_e::PREPROC)) != nullptr)
       {
          if (  (chunk_is_token(pc, CT_SEMICOLON) && hit_semicolon)
@@ -1020,10 +1221,12 @@ static void check_template(chunk_t *start)
          {
             break;
          }
+
          if (chunk_is_token(pc, CT_SEMICOLON) && !hit_semicolon)
          {
             hit_semicolon = true;
          }
+
          if (  ((  chunk_is_token(pc, CT_IF)
                 || chunk_is_token(pc, CT_RETURN)
                 || chunk_is_token(pc, CT_WHILE)
@@ -1034,7 +1237,6 @@ static void check_template(chunk_t *start)
             break;
          }
       }
-
       /*
        * Scan forward to the angle close
        * If we have a comparison in there, then it can't be a template.
@@ -1044,6 +1246,7 @@ static void check_template(chunk_t *start)
       size_t    num_tokens = 1;
 
       tokens[0] = CT_ANGLE_OPEN;
+
       for (pc = chunk_get_next_ncnl(start, scope_e::PREPROC);
            pc != nullptr;
            pc = chunk_get_next_ncnl(pc, scope_e::PREPROC))
@@ -1051,12 +1254,14 @@ static void check_template(chunk_t *start)
          LOG_FMT(LTEMPL, "%s(%d): type is %s, num_tokens is %zu\n",
                  __func__, __LINE__, get_token_name(pc->type), num_tokens);
 
+         log_rule_B("tok_split_gte");
+
          if (  (tokens[num_tokens - 1] == CT_ANGLE_OPEN)
             && (pc->str[0] == '>')
             && (pc->len() > 1)
             && (  options::tok_split_gte()
                || (  (chunk_is_str(pc, ">>", 2) || chunk_is_str(pc, ">>>", 3))
-                  && num_tokens >= 2)))
+                  && (num_tokens >= 2 || (num_tokens >= 1 && in_type_cast)))))
          {
             LOG_FMT(LTEMPL, "%s(%d): {split '%s' at orig_line %zu, orig_col %zu}\n",
                     __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
@@ -1106,19 +1311,29 @@ static void check_template(chunk_t *start)
          }
          else if (chunk_is_token(pc, CT_PAREN_CLOSE))
          {
+            if (num_tokens == 0)
+            {
+               fprintf(stderr, "%s(%d): num_tokens is ZERO, cannot be decremented, at line %zu, column %zu\n",
+                       __func__, __LINE__, pc->orig_line, pc->orig_col);
+               log_flush(true);
+               exit(EX_SOFTWARE);
+            }
             num_tokens--;
+
             if (tokens[num_tokens] != CT_PAREN_OPEN)
             {
                break;  // unbalanced parentheses
             }
          }
       }
+
       end = pc;
    }
 
    if (chunk_is_token(end, CT_ANGLE_CLOSE))
    {
       pc = chunk_get_next_ncnl(end, scope_e::PREPROC);
+
       if (pc == nullptr || pc->type != CT_NUMBER)
       {
          LOG_FMT(LTEMPL, "%s(%d): Template detected\n", __func__, __LINE__);
@@ -1135,7 +1350,6 @@ static void check_template(chunk_t *start)
          return;
       }
    }
-
    LOG_FMT(LTEMPL, "%s(%d): - Not a template: end = %s\n",
            __func__, __LINE__, (end != NULL) ? get_token_name(end->type) : "<null>");
    set_chunk_type(start, CT_COMPARE);
@@ -1162,16 +1376,19 @@ static void check_template_arg(chunk_t *start, chunk_t *end)
    // 1. run to test if expression is numeric
    bool    expressionIsNumeric = false;
    chunk_t *pc                 = start;
+
    while (pc != end)
    {
       chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
       // a test "if (next == nullptr)" is not necessary
       chunk_flags_set(pc, PCF_IN_TEMPLATE);
+
       if (chunk_is_token(pc, CT_DECLTYPE) || chunk_is_token(pc, CT_SIZEOF))
       {
          expressionIsNumeric = true;
          break;
       }
+
       if (next->type != CT_PAREN_OPEN)
       {
          if (  chunk_is_token(pc, CT_NUMBER)
@@ -1185,15 +1402,18 @@ static void check_template_arg(chunk_t *start, chunk_t *end)
    }
    LOG_FMT(LTEMPL, "%s(%d): expressionIsNumeric is %s\n",
            __func__, __LINE__, expressionIsNumeric ? "TRUE" : "FALSE");
+
    // 2. run to do the work
    if (!expressionIsNumeric)
    {
       pc = start;
+
       while (pc != end)
       {
          chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
          // a test "if (next == nullptr)" is not necessary
          chunk_flags_set(pc, PCF_IN_TEMPLATE);
+
          if (next->type != CT_PAREN_OPEN)
          {
             make_type(pc);
@@ -1218,6 +1438,7 @@ static void check_template_args(chunk_t *start, chunk_t *end)
       switch (pc->type)
       {
       case CT_COMMA:
+
          if (tokens.empty())
          {
             // Check current argument
@@ -1232,6 +1453,7 @@ static void check_template_args(chunk_t *start, chunk_t *end)
          break;
 
       case CT_ANGLE_CLOSE:
+
          if (!tokens.empty() && tokens.back() == CT_ANGLE_OPEN)
          {
             tokens.pop_back();
@@ -1239,6 +1461,7 @@ static void check_template_args(chunk_t *start, chunk_t *end)
          break;
 
       case CT_PAREN_CLOSE:
+
          if (!tokens.empty() && tokens.back() == CT_PAREN_OPEN)
          {
             tokens.pop_back();
@@ -1266,19 +1489,21 @@ static void cleanup_objc_property(chunk_t *start)
       LOG_FMT(LTEMPL, "%s(%d): Property is not followed by openning paren\n", __func__, __LINE__);
       return;
    }
-
    set_chunk_parent(open_paren, start->type);
 
    chunk_t *tmp = chunk_get_next_type(start, CT_PAREN_CLOSE, start->level);
+
    if (tmp != NULL)
    {
       set_chunk_parent(tmp, start->type);
       tmp = chunk_get_next_ncnl(tmp);
+
       if (tmp != NULL)
       {
          chunk_flags_set(tmp, PCF_STMT_START | PCF_EXPR_START);
 
          tmp = chunk_get_next_type(tmp, CT_SEMICOLON, start->level);
+
          if (tmp != NULL)
          {
             set_chunk_parent(tmp, start->type);
@@ -1302,13 +1527,14 @@ static void mark_selectors_in_property_with_open_paren(chunk_t *open_paren)
          && (chunk_is_str(tmp, "setter", 6) || chunk_is_str(tmp, "getter", 6)))
       {
          tmp = tmp->next;
+
          while (  tmp
                && tmp->type != CT_COMMA
                && tmp->type != CT_PAREN_CLOSE)
          {
             if (chunk_is_token(tmp, CT_WORD) || chunk_is_str(tmp, ":", 1))
             {
-               tmp->type = CT_OC_SEL_NAME;
+               set_chunk_type(tmp, CT_OC_SEL_NAME);
             }
             tmp = tmp->next;
          }
@@ -1332,9 +1558,8 @@ static void mark_attributes_in_property_with_open_paren(chunk_t *open_paren)
       if (  (chunk_is_token(tmp, CT_COMMA) || chunk_is_token(tmp, CT_PAREN_OPEN))
          && (chunk_is_token(tmp->next, CT_WORD) || chunk_is_token(tmp->next, CT_TYPE)))
       {
-         tmp->next->type = CT_OC_PROPERTY_ATTR;
+         set_chunk_type(tmp->next, CT_OC_PROPERTY_ATTR);
       }
       tmp = tmp->next;
    }
 }
-
