@@ -9,14 +9,18 @@
 #ifndef UNC_TOOLS_H_INCLUDED
 #define UNC_TOOLS_H_INCLUDED
 
-//#define DEVELOP_ONLY
-#ifdef DEVELOP_ONLY
+#include "chunk_list.h"
 #include "prototypes.h"
 #include "uncrustify_types.h"
-#include "chunk_list.h"
+
+#if defined DEBUG
+#define PROT_THE_LINE    prot_the_line(__func__, __LINE__, 0, 0);
+#else
+#define PROT_THE_LINE    /* do nothing */;
+#endif
 
 
-void prot_the_line(int theLine, unsigned int actual_line);
+void prot_the_line(const char *func_name, int theLine, unsigned int actual_line, size_t partNumber);
 
 
 void prot_the_source(int theLine);
@@ -33,5 +37,6 @@ void dump_out(unsigned int type);
 void dump_in(unsigned int type);
 
 
-#endif /* DEVELOP_ONLY */
+size_t get_A_Number();
+
 #endif /* UNC_TOOLS_H_INCLUDED */
