@@ -6,25 +6,29 @@
  * @author  Ben Gardner
  * @license GPL v2+
  */
+
 #ifndef UNCRUSTIFY_TYPES_H_INCLUDED
 #define UNCRUSTIFY_TYPES_H_INCLUDED
 
-#include <vector>
-#include <deque>
-
 #include "base_types.h"
-#include "options.h"
-#include "option_enum.h"
-#include "token_enum.h"    // c_token_t
+#include "enum_flags.h"
 #include "log_levels.h"
 #include "logger.h"
+#include "option_enum.h"
+#include "options.h"
+#include "pcf_flags.h"
+#include "token_enum.h"    // c_token_t
 #include "unc_text.h"
-#include <cstdio>
+
 #include <assert.h>
+#include <cstdio>
+#include <deque>
+#include <vector>
 #ifdef HAVE_UTIME_H
 #include <utime.h>
 #endif
 
+
 class ParseFrame;
 
 
@@ -104,103 +108,6 @@ struct indent_ptr_t
 };
 
 
-#define PCF_BIT(b)    (1ULL << b)
-
-// Copy flags are in the lower 16 bits
-#define PCF_COPY_FLAGS         0x0000ffff
-#define PCF_IN_PREPROC         PCF_BIT(0)  //! in a preprocessor
-#define PCF_IN_STRUCT          PCF_BIT(1)  //! in a struct
-#define PCF_IN_ENUM            PCF_BIT(2)  //! in enum
-#define PCF_IN_FCN_DEF         PCF_BIT(3)  //! inside function def parens
-#define PCF_IN_FCN_CALL        PCF_BIT(4)  //! inside function call parens
-#define PCF_IN_SPAREN          PCF_BIT(5)  //! inside for/if/while/switch parens
-#define PCF_IN_TEMPLATE        PCF_BIT(6)
-#define PCF_IN_TYPEDEF         PCF_BIT(7)
-#define PCF_IN_CONST_ARGS      PCF_BIT(8)
-#define PCF_IN_ARRAY_ASSIGN    PCF_BIT(9)
-#define PCF_IN_CLASS           PCF_BIT(10)
-#define PCF_IN_CLASS_BASE      PCF_BIT(11)
-#define PCF_IN_NAMESPACE       PCF_BIT(12)
-#define PCF_IN_FOR             PCF_BIT(13)
-#define PCF_IN_OC_MSG          PCF_BIT(14)
-#define PCF_IN_WHERE_SPEC      PCF_BIT(15)  /* inside C# 'where' constraint clause on class or function def */
-
-// Non-Copy flags are in the upper 48 bits
-#define PCF_FORCE_SPACE        PCF_BIT(16)  //! must have a space after this token
-#define PCF_STMT_START         PCF_BIT(17)  //! marks the start of a statement
-#define PCF_EXPR_START         PCF_BIT(18)
-#define PCF_DONT_INDENT        PCF_BIT(19)  //! already aligned!
-#define PCF_ALIGN_START        PCF_BIT(20)
-#define PCF_WAS_ALIGNED        PCF_BIT(21)
-#define PCF_VAR_TYPE           PCF_BIT(22)  //! part of a variable def type
-#define PCF_VAR_DEF            PCF_BIT(23)  //! variable name in a variable def
-#define PCF_VAR_1ST            PCF_BIT(24)  //! 1st variable def in a statement
-#define PCF_VAR_1ST_DEF        (PCF_VAR_DEF | PCF_VAR_1ST)
-#define PCF_VAR_INLINE         PCF_BIT(25)  //! type was an inline struct/enum/union
-#define PCF_RIGHT_COMMENT      PCF_BIT(26)
-#define PCF_OLD_FCN_PARAMS     PCF_BIT(27)
-#define PCF_LVALUE             PCF_BIT(28)  //! left of assignment
-#define PCF_ONE_LINER          PCF_BIT(29)
-#define PCF_ONE_CLASS          (PCF_ONE_LINER | PCF_IN_CLASS)
-#define PCF_EMPTY_BODY         PCF_BIT(30)
-#define PCF_ANCHOR             PCF_BIT(31)  //! aligning anchor
-#define PCF_PUNCTUATOR         PCF_BIT(32)
-#define PCF_INSERTED           PCF_BIT(33)  //! chunk was inserted from another file
-#define PCF_LONG_BLOCK         PCF_BIT(34)  //! the block is 'long' by some measure
-#define PCF_OC_BOXED           PCF_BIT(35)  //! inside OC boxed expression
-#define PCF_KEEP_BRACE         PCF_BIT(36)  //! do not remove brace
-#define PCF_OC_RTYPE           PCF_BIT(37)  //! inside OC return type
-#define PCF_OC_ATYPE           PCF_BIT(38)  //! inside OC arg type
-#define PCF_WF_ENDIF           PCF_BIT(39)  //! #endif for whole file ifdef
-#define PCF_IN_QT_MACRO        PCF_BIT(40)  //! in a QT-macro, i.e. SIGNAL, SLOT
-
-#ifdef DEFINE_PCF_NAMES
-static const char *pcf_names[] =
-{
-   "IN_PREPROC",        // 0
-   "IN_STRUCT",         // 1
-   "IN_ENUM",           // 2
-   "IN_FCN_DEF",        // 3
-   "IN_FCN_CALL",       // 4
-   "IN_SPAREN",         // 5
-   "IN_TEMPLATE",       // 6
-   "IN_TYPEDEF",        // 7
-   "IN_CONST_ARGS",     // 8
-   "IN_ARRAY_ASSIGN",   // 9
-   "IN_CLASS",          // 10
-   "IN_CLASS_BASE",     // 11
-   "IN_NAMESPACE",      // 12
-   "IN_FOR",            // 13
-   "IN_OC_MSG",         // 14
-   "#15",               // 15
-   "FORCE_SPACE",       // 16
-   "STMT_START",        // 17
-   "EXPR_START",        // 18
-   "DONT_INDENT",       // 19
-   "ALIGN_START",       // 20
-   "WAS_ALIGNED",       // 21
-   "VAR_TYPE",          // 22
-   "VAR_DEF",           // 23
-   "VAR_1ST",           // 24
-   "VAR_INLINE",        // 25
-   "RIGHT_COMMENT",     // 26
-   "OLD_FCN_PARAMS",    // 27
-   "LVALUE",            // 28
-   "ONE_LINER",         // 29
-   "EMPTY_BODY",        // 30
-   "ANCHOR",            // 31
-   "PUNCTUATOR",        // 32
-   "INSERTED",          // 33
-   "LONG_BLOCK",        // 34
-   "OC_BOXED",          // 35
-   "KEEP_BRACE",        // 36
-   "OC_RTYPE",          // 37
-   "OC_ATYPE",          // 38
-   "WF_ENDIF",          // 39
-   "IN_QT_MACRO",       // 40
-};
-#endif
-
 struct align_ptr_t
 {
    chunk_t *next;       //! nullptr or the chunk that should be under this one
@@ -234,15 +141,16 @@ struct chunk_t
    {
       memset(&align, 0, sizeof(align));
       memset(&indent, 0, sizeof(indent));
-      next          = 0;
-      prev          = 0;
+      next          = nullptr;
+      prev          = nullptr;
+      parent        = nullptr;
       type          = CT_NONE;
       parent_type   = CT_NONE;
       orig_line     = 0;
       orig_col      = 0;
       orig_col_end  = 0;
       orig_prev_sp  = 0;
-      flags         = 0;
+      flags         = PCF_NONE;
       column        = 0;
       column_indent = 0;
       nl_count      = 0;
@@ -270,15 +178,17 @@ struct chunk_t
 
    chunk_t      *next;            //! pointer to next chunk in list
    chunk_t      *prev;            //! pointer to previous chunk in list
+   chunk_t      *parent;          //! pointer to parent chunk(not always set)
    align_ptr_t  align;
    indent_ptr_t indent;
    c_token_t    type;             //! type of the chunk itself
    c_token_t    parent_type;      //! type of the parent chunk usually CT_NONE
+                                  //! might be different from parent->parent_type (above)
    size_t       orig_line;        //! line number of chunk in input file
    size_t       orig_col;         //! column where chunk started in the input file, is always > 0
    size_t       orig_col_end;     //! column where chunk ended in the input file, is always > 1
    UINT32       orig_prev_sp;     //! whitespace before this token
-   UINT64       flags;            //! see PCF_xxx
+   pcf_flags_t  flags;            //! see PCF_xxx
    size_t       column;           //! column of chunk
    size_t       column_indent;    /** if 1st on a line, set to the 'indent'
                                    * column, which may be less than the real
