@@ -29,3 +29,30 @@ class E
 public:
     E();
 };
+namespace F {
+}
+void foo();
+class G
+{
+};
+void bar();
+
+void foo2();
+namespace E
+{
+}
+void bar2();
+
+void foo3();
+namespace F
+{
+}
+
+void bar3();
+
+void foo4();
+class I
+{
+};
+
+void bar4();
